# Representing a scalar

# You can construct and manipulate a population of neurons (ensemble) in nengo. 
# This model shows how the activity of neural populations can be thought 
# of as representing a mathematical variable (a scalar value).

# The model has paramenters as described in the book and uses a single 
# population (ensemble) of 100 LIF neurons. Note that the default max rates 
# in Nengo 2.0 are (200, 400), so you have to explicitly specify them to be 
# (100, 200) to create the model with the same parameters as described in the 
# book. Moreover the 'Node Factory' feature of ensembles mentioned in the book 
# maps to the 'neuron_type' in Nengo 2.0 which is set to LIF by default. The 
# default values of tauRC, tauRef, radius and intercepts in Nengo 2.0 are the 
# same as those mentioned in the book.

# Press the play button to run the simulation.
# The graph on the top left shows the input and the graph on the top right 
# shows the the decoded value of the neural spiking (a linearly decoded estimate 
# of the input). The graph on the bottom right shows the spike raster which is 
# the spiking output of the neuron population (x).

#Setup the environment
import numpy as np
import nengo
from nengo.dists import Uniform

model = nengo.Network(label='Many Neurons')
with model:
    # Input sine wave with range 1, freq of 16 rad/s
    input = nengo.Node(lambda t: np.sin(16 * t))  
    # Input sine wave with range increased to 4
    #input = nengo.Node(lambda t: 4*np.sin(16 * t))   
    
    #Ensemble with 100 LIF neurons  
    x = nengo.Ensemble(100, dimensions=1, max_rates = Uniform(100, 200))
    
    #Connecting input to ensemble
    nengo.Connection(input, x) 



# Increasing the range of Input

# You have seen that the population of neurons does a reasonably good job of 
# representing the input. However, neurons cannot represent arbitrary values 
# well and you can verify this by increasing the range of the input to 4 
# ( input = nengo.Node(lambda t: 4np.sin(16 t)) ). You will observe the same 
# saturation effects as described in the book, showing that the neurons do a 
# much better job at representing information within the defined radius.



import os
import time

import nengo_gui  # Imported to find project_root
from nengo_gui import testing_tools as tt


def download_file(driver, name_of_file):
    time.sleep(1)
    file_name = driver.find_element_by_xpath('//*[@id="filename"]')
    file_name.click()
    time.sleep(0.25)

    field = driver.find_element_by_xpath('//*[@id="save-as-filename"]')
    time.sleep(0.3)
    field.clear()
    time.sleep(0.25)
    field.send_keys(name_of_file)
    time.sleep(0.2)

    accept = driver.find_element_by_xpath('//*[@id="OK"]')
    accept.click()

    time.sleep(5)


def test_save_as(driver):
    # Saves the default.py file as test_download, tests whether it downloaded
    # correctly.
    tt.reset_page(driver)
    download_file(driver, "nengo_gui/examples/test_download.py")

    # Finds files to be read
    project_root = os.path.dirname(nengo_gui.__file__)
    test_file = os.path.join(project_root, "examples", "test_download.py")
    test_file_conf = os.path.join(
        project_root, "examples", "test_download.py.cfg")
    default_file = os.path.join(project_root, "examples", "default.py")

    assert(os.path.isfile(test_file))
    with open(test_file, 'r') as f1:
        test_code = f1.read()
    with open(default_file, 'r') as f2:
        default_code = f2.read()

    assert test_code == default_code

    # Deletes downloaded files
    os.remove(test_file)
    os.remove(test_file_conf)

    download_file(driver, "nengo_gui/examples/default.py")
    time.sleep(2)
    try:
        alert = driver.switch_to_alert()
        alert.accept()
    except:
        assert False, "No warning present"



#!/usr/bin/env python
#
# This code is part of Grandalf
# Copyright (C) 2008 Axel Tillequin (bdcht3@gmail.com) and others
# published under GPLv2 license or EPLv1 license
# Contributor(s): Axel Tillequin, Fabio Zadrozny

from  numpy import array
from  math  import atan2,cos,sin,sqrt
import math, numpy

try:
    import ply.lex as lex
    import ply.yacc as yacc
    _has_ply = True
except ImportError:
    _has_ply = False

#------------------------------------------------------------------------------
class  Poset(object):

    def __init__(self,L):
        self.o = []
        s = set()
        for obj in L:
            if not obj in s:
                self.o.append(obj)
                s.add(obj)
            else:
                print('warning: obj was already added in poset at index %d' \
                      %s.index(obj))
        self.s = s

    def __repr__(self):
        return 'poset(%r)' % (self.o,)

    def __str__(self):
        f='%%%dd'%len(str(len(self.o)))
        s=[]
        for i,x in enumerate(self.o):
            s.append(f%i+'.| %s'%repr(x))
        return '\n'.join(s)

    def add(self,obj):
        if obj not in self.s:
            self.o.append(obj)
            self.s.add(obj)

    def remove(self,obj):
        if obj in self.s:
            self.o.remove(obj)
            self.s.remove(obj)

    def index(self,obj):
        return self.o.index(obj)

    def __len__(self):
        return len(self.o)

    def __iter__(self):
        for obj in self.o:
            yield obj

    def __cmp__(self,other):
        return cmp(other.s,self.s)

    def __eq__(self,other):
        return other.s==self.s

    def __ne__(self,other):
        return other.s!=self.s

    def copy(self):
        return Poset(self.o)

    __copy__ = copy
    def deepcopy(self):
        from copy import deepcopy
        L = deepcopy(self.o)
        return Poset(L)

    def __or__(self,other):
        return self.union(other)

    def union(self,other):
        return Poset(self.o+other.o)

    def update(self,other):
        for obj in other:
            if obj not in self:
                self.add(obj)

    def __and__(self,other):
        return self.intersection(other)

    def intersection(self,other):
        return Poset(self.s.intersection(other.s))

    def __xor__(self,other):
        return self.symmetric_difference(other)

    def symmetric_difference(self,other):
        return Poset(self.s^other.s)

    def __sub__(self,other):
        return self.difference(other)

    def difference(self,other):
        return Poset(self.s-other.s)

    def __contains__(self,obj):
        return (obj in self.s)

    def issubset(self,other):
        return (self.s.issubset(other.s))

    def issuperset(self,other):
        return self.s.issuperset(other.s)

    __le__ = issubset
    __ge__ = issuperset

    def __lt__(self,other):
        return (self<=other and len(self)!=len(other))

    def __gt__(self,other):
        return (self>=other and len(self)!=len(other))


#  rand_ortho1 returns a numpy.array representing
#  a random normalized n-dimension vector orthogonal to (1,1,1,...,1).
def  rand_ortho1(n):
    from random import SystemRandom
    r = SystemRandom()
    pos = [r.random() for x in xrange(n)]
    s = sum(pos)

    # Note: returning only positive values (if we had only negative values
    # an exception such as:
    # inf X:\grandalf\grandalf\layouts.py:784: RuntimeWarning: divide by zero encountered in double_scalars
    #   sfactor = 1.0/max(y.max(),x.max())
    # Could be found (randomly happening in test-layouts:test_splines).
    v = abs(array(pos,dtype=float)-(s/len(pos)))
    norm = sqrt(sum(v*v))
    return v/norm


#------------------------------------------------------------------------------
#TODO:  this was imported here from masr, but since we have
#  here access to numpy.array, we could use it for vectors operations.
def  intersect2lines(p1,p2,p3,p4):
    x1,y1 = p1
    x2,y2 = p2
    x3,y3 = p3
    x4,y4 = p4
    b = (x2-x1,y2-y1)
    d = (x4-x3,y4-y3)
    det = b[0]*d[1] - b[1]*d[0]
    if det==0: return None
    c = (x3-x1,y3-y1)
    t = float(c[0]*b[1] - c[1]*b[0])/(det*1.)
    if (t<0. or t>1.): return None
    t = float(c[0]*d[1] - c[1]*d[0])/(det*1.)
    if (t<0. or t>1.): return None
    x = x1 + t*b[0]
    y = y1 + t*b[1]
    return (x,y)


#------------------------------------------------------------------------------
#  intersectR returns the intersection point between the Rectangle
#  (w,h) that characterize the view object and the line that goes
#  from the views' object center to the 'topt' point.
def  intersectR(view,topt):
    # we compute intersection in local views' coord:
    # center of view is obviously :
    x1,y1 = 0,0
    # endpoint in view's coord:
    x2,y2 = topt[0]-view.xy[0],topt[1]-view.xy[1]
    # bounding box:
    bbx2 = view.w/2
    bbx1 = -bbx2
    bby2 = view.h/2
    bby1 = -bby2
    # all 4 segments of the bb:
    S = [((x1,y1),(x2,y2),(bbx1,bby1),(bbx2,bby1)),
              ((x1,y1),(x2,y2),(bbx2,bby1),(bbx2,bby2)),
              ((x1,y1),(x2,y2),(bbx1,bby2),(bbx2,bby2)),
              ((x1,y1),(x2,y2),(bbx1,bby2),(bbx1,bby1))]
    # check intersection with each seg:
    for segs in S:
        xy = intersect2lines(*segs)
        if xy!=None:
            x,y = xy
            # return global coord:
            x += view.xy[0]
            y += view.xy[1]
            return (x,y)
    # there can't be no intersection unless the endpoint was
    # inside the bb !
    raise ValueError('no intersection found (point inside ?!). view: %s topt: %s' % (view, topt))


#------------------------------------------------------------------------------
def  getangle(p1,p2):
    x1,y1 = p1
    x2,y2 = p2
    theta = atan2(y2-y1,x2-x1)
    return theta


#------------------------------------------------------------------------------
#  intersectC returns the intersection point between the Circle
#  of radius r and centered on views' position with the line
#  to the 'topt' point.
def  intersectC(view, r, topt):
    theta = getangle(view.xy,topt)
    x = int(cos(theta)*r)
    y = int(sin(theta)*r)
    return (x,y)


def median_wh(views):
    mw = [v.w for v in views]
    mh = [v.h for v in views]
    mw.sort()
    mh.sort()
    return (mw[len(mw)//2],mh[len(mh)//2])

#------------------------------------------------------------------------------
#  setcurve returns the spline curve that path through the list of points P.
#  The spline curve is a list of cubic bezier curves (nurbs) that have
#  matching tangents at their extreme points.
#  The method considered here is taken from "The NURBS book" (Les A. Piegl,
#  Wayne Tiller, Springer, 1997) and implements a local interpolation rather
#  than a global interpolation.
def setcurve(e,pts,tgs=None):
    P = map(array,pts)
    n = len(P)
    # tangent estimation
    if tgs:
      assert len(tgs)==n
      T = map(array,tgs)
      Q = [ P[k+1]-P[k] for k in range(0,n-1)]
    else:
      Q,T = tangents(P,n)
    splines=[]
    for k in xrange(n-1):
        t = T[k]+T[k+1]
        a = 16. - (t.dot(t))
        b = 12.*(Q[k].dot(t))
        c = -36. * Q[k].dot(Q[k])
        D = (b*b) - 4.*a*c
        assert D>=0
        sd = sqrt(D)
        s1,s2 = (-b-sd)/(2.*a),(-b+sd)/(2.*a)
        s = s2
        if s1>=0: s=s1
        C0 = tuple(P[k])
        C1 = tuple(P[k] + (s/3.)*T[k])
        C2 = tuple(P[k+1] -(s/3.)*T[k+1])
        C3 = tuple(P[k+1])
        splines.append([C0,C1,C2,C3])
    return splines

def tangents(P,n):
    assert n>=2
    Q = []
    T = []
    for k in xrange(0,n-1):
        q = P[k+1]-P[k]
        t = q/sqrt(q.dot(q))
        Q.append(q)
        T.append(t)
    T.append(t)
    return (Q,T)

#------------------------------------------------------------------------------
def setroundcorner(e,pts):
    P = map(array,pts)
    n = len(P)
    Q,T = tangents(P,n)
    c0 = P[0]
    t0 = T[0]
    k0 = 0
    splines = []
    k  = 1
    while k<n:
        z = abs(t0[0]*T[k][1]-(t0[1]*T[k][0]))
        if z<1.e-6:
            k+=1
            continue
        if (k-1)>k0: splines.append([c0,P[k-1]])
        if (k+1)<n:
            splines.extend(setcurve(e,[P[k-1],P[k+1]],tgs=[T[k-1],T[k+1]]))
        else:
            splines.extend(setcurve(e,[P[k-1],P[k]],tgs=[T[k-1],T[k]]))
            break
        if (k+2)<n:
            c0 = P[k+1]
            t0 = T[k+1]
            k0 = k+1
            k+=2
        else:
            break
    return splines or [[P[0],P[-1]]]

#------------------------------------------------------------------------------
# LALR(1) parser for Graphviz dot file format.
class Dot:

    _reserved = (
        'strict',
        'graph',
        'digraph',
        'subgraph',
        'node',
        'edge',
    )
    _tokens = (
        'regulars',
        'string',
        'html',
        'comment',
    )+_reserved

    _literals = [',',';','-','>','=',':','[',']','{','}']

    class Lexer(object):
        def __init__(self):
            self.whitespace = '\0\t\n\f\r '
            self.reserved = Dot._reserved
            self.tokens = Dot._tokens
            self.literals = Dot._literals
            self.t_ignore = self.whitespace

        def t_regulars(self,t):
            r'[-]?[\w.]+'
            v = t.value.lower()
            if v in self.reserved:
                t.type = v
                return t
            # check numeric string
            if v[0].isdigit() or v[0] in ['-','.']:
                try:
                    float(v)
                except ValueError:
                    print('invalid numeral token: %s'%v)
                    raise SyntaxError
            elif '.' in v: # forbidden in non-numeric
                raise SyntaxError
            return t

        def t_comment_online(self,t):
            r'(//(.*)\n)|\\\n'
            pass

        def t_comment_macro(self,t):
            r'(\#(.*)\n)'
            pass

        def t_comment_multline(self,t):
            r'(/\*)'
            start=t.lexer.lexpos
            t.lexer.lexpos = t.lexer.lexdata.index('*/',start)+2

        def t_string(self,t):
            r'"'
            start=t.lexer.lexpos-1
            i = t.lexer.lexdata.index('"',start+1)
            while t.lexer.lexdata[i-1] =='\\' :
                i = t.lexer.lexdata.index('"',i+1)
            t.value = t.lexer.lexdata[start:i+1]
            t.lexer.lexpos = i+1
            return t

        def t_html(self,t):
            r'<'
            start=t.lexer.lexpos-1
            level=1
            i=start+1
            while level>0:
                c = t.lexer.lexdata[i]
                if c=='<': level += 1
                if c=='>': level -= 1
                i += 1
            t.value = t.lexer.lexdata[start:i]
            t.lexer.lexpos = i
            return t

        def t_ANY_error(self,t):
            print("Illegal character '%s'" % t.value[0])
            t.lexer.skip(1)

        def build(self,**kargs):
            if _has_ply:
                self._lexer = lex.lex(module=self, **kargs)

        def test(self,data):
            self._lexer.input(data)
            while 1:
                tok = self._lexer.token()
                if not tok:
                    break
                print(tok)

    # Classes for the AST returned by Parser:
    class graph(object):
        def __init__(self,name,data,strict=None,direct=None):
            self.name = name
            self.strict = strict
            self.direct = direct
            self.nodes = {}
            self.edges = []
            self.subgraphs = []
            self.attr = {}
            eattr = {}
            nattr = {}
            for x in data: # data is a statements (list of stmt)
                # x is a stmt, ie one of:
                # a graph object (subgraph)
                # a attr object (graph/node/edge attributes)
                # a dict object (ID=ID)
                # a node object
                # a list of edges
                if isinstance(x,Dot.graph):
                    self.subgraphs.append(x)
                elif isinstance(x,Dot.attr):
                    if x.type=='graph':
                        self.attr.update(x.D)
                    elif x.type=='node' :
                        nattr.update(x.D)
                    elif x.type=='edge' :
                        eattr.update(x.D)
                    else :
                        raise TypeError('invalid attribute type')
                elif isinstance(x,dict):
                    self.attr.update(x)
                elif isinstance(x,Dot.node):
                    x.attr.update(nattr)
                    self.nodes[x.name] = x
                else:
                    for e in x:
                        e.attr.update(eattr)
                        self.edges.append(e)
                        for n in [e.n1,e.n2]:
                            if isinstance(n,Dot.graph): continue
                            if n.name not in self.nodes:
                                n.attr.update(nattr)
                                self.nodes[n.name] = n

        def __repr__(self):
            u = u'<%s instance at %x, name: %s, %d nodes>'%(
                   self.__class__,
                   id(self),
                   self.name,
                   len(self.nodes))
            return u.encode('utf-8')

    class attr(object):
        def __init__(self,type,D):
            self.type=type
            self.D = D

    class edge(object):
        def __init__(self,n1,n2):
            self.n1 = n1
            self.n2 = n2
            self.attr = {}

    class node(object):
        def __init__(self,name,port=None):
            self.name = name
            self.port = port
            self.attr = {}

    class Parser(object):
        def __init__(self):
            self.tokens = Dot._tokens

        def __makelist(self,p):
            N=len(p)
            if N>2:
                L = p[1]
                L.append(p[N-1])
            else:
                L = []
                if N>1:
                    L.append(p[N-1])
            p[0] = L

        def p_Data(self,p):
            '''Data : Data Graph
                    | Graph'''
            self.__makelist(p)

        def p_Graph_strict(self,p):
            '''Graph : strict graph name Block'''
            p[0] = Dot.graph(name=p[3],data=p[4],strict=1,direct=0)
            #print 'Dot.Parser: graph object %s created'%p[0].name

        def p_Graph_graph(self,p):
            '''Graph : graph name Block'''
            p[0] = Dot.graph(name=p[2],data=p[3],strict=0,direct=0)

        def p_Graph_strict_digraph(self,p):
            '''Graph : strict digraph name Block'''
            p[0] = Dot.graph(name=p[3],data=p[4],strict=1,direct=1)

        def p_Graph_digraph(self,p):
            '''Graph : digraph name Block'''
            p[0] = Dot.graph(name=p[2],data=p[3],strict=0,direct=1)

        def p_ID(self,p):
            '''ID : regulars
                  | string
                  | html '''
            p[0] = p[1]

        def p_name(self,p):
            '''name : ID
                    | '''
            if len(p)==1:
                p[0]=''
            else:
                p[0]=p[1]

        def p_Block(self,p):
            '''Block : '{' statements '}' '''
            p[0] = p[2]

        def p_statements(self,p):
            '''statements : statements stmt
                          | stmt
                          | '''
            self.__makelist(p)

        def p_stmt(self,p):
            '''stmt : stmt ';' '''
            p[0] = p[1]

        def p_comment(self,p):
            '''stmt : comment'''
            pass  # comment tokens are not outputed by lexer anyway

        def p_stmt_sub(self,p):
            '''stmt : sub'''
            p[0] = p[1]

        def p_subgraph(self,p):
            '''sub : subgraph name Block
                   | Block '''
            N = len(p)
            if N>2:
                ID = p[2]
            else:
                ID = ''
            p[0] = Dot.graph(name=ID,data=p[N-1],strict=0,direct=0)

        def p_stmt_assign(self,p):
            '''stmt : affect '''
            p[0] = p[1]

        def p_affect(self,p):
            '''affect : ID '=' ID '''
            p[0] = dict([(p[1],p[3])])

        def p_stmt_lists(self,p):
            '''stmt : graph attrs
                    | node  attrs
                    | edge  attrs '''
            p[0] = Dot.attr(p[1],p[2])

        def p_attrs(self,p):
            '''attrs : attrs attrl
                     | attrl '''
            if len(p)==3:
                p[1].update(p[2])
            p[0] = p[1]

        def p_attrl(self,p):
            '''attrl : '[' alist ']' '''
            L={}
            for a in p[2]:
                if isinstance(a,dict):
                    L.update(a)
                else:
                    L[a] = 'true'
            p[0] = L

        def p_alist_comma(self,p):
            '''alist : alist ',' alist '''
            p[1].extend(p[3])
            p[0] = p[1]

        def p_alist_affect(self,p):
            '''alist : alist affect
                     | alist ID
                     | affect
                     | ID
                     | '''
            self.__makelist(p)

        def p_stmt_E_attrs(self,p):
            '''stmt : E attrs '''
            for e in p[1]: e.attr = p[2]
            p[0] = p[1]

        def p_stmt_N_attrs(self,p):
            '''stmt : N attrs '''
            p[1].attr = p[2]
            p[0] = p[1]

        def p_stmt_EN(self,p):
            '''stmt : E
                    | N '''
            p[0] = p[1]

        def p_E(self,p):
            '''E : E   link
                 | elt link '''
            try:
                L = p[1]
                L.append(Dot.edge(L[-1].n2,p[2]))
            except:
                L = []
                L.append(Dot.edge(p[1],p[2]))
            p[0] = L

        def p_elt(self,p):
            '''elt : N
                   | sub '''
            p[0] = p[1]

        def p_link(self,p):
            '''link : '-' '>' elt
                    | '-' '-' elt '''
            p[0] = p[3]

        def p_N_port(self,p):
            '''N : ID port '''
            p[0] = Dot.node(p[1],port=p[2])

        def p_N(self,p):
            '''N : ID '''
            p[0] = Dot.node(p[1])

        def p_port(self,p):
            '''port : ':' ID '''
            p[0] = p[2]

        def p_port2(self,p):
            '''port : port port'''
            assert p[2] in ['n','ne',
                            'e','se',
                            's','sw',
                            'w','nw',
                            'c','_']
            p[0] = "%s:%s"%(p[1],p[2])

        def p_error(self,p):
            print('Syntax Error: %s' % (p,))
            self._parser.restart()

        def build(self,**kargs):
            opt=dict(debug=0,write_tables=0)
            opt.update(**kargs)
            if _has_ply:
                self._parser = yacc.yacc(module=self,**opt)

    def __init__(self,**kargs):
        self.lexer  = Dot.Lexer()
        self.parser = Dot.Parser()
        if not _has_ply:
            print('warning: Dot parser not supported (install python-ply)')

    def parse(self,data):
        try:
            self.parser._parser.restart()
        except AttributeError:
            self.lexer.build(reflags=lex.re.UNICODE)
            self.parser.build()
        except:
            print('unexpected error')
            return None
        try:
            s = data.decode('utf-8')
        except UnicodeDecodeError:
            s = data
        L=self.parser._parser.parse(s,
                                    lexer=self.lexer._lexer)
        return L

    def read(self,filename):
        f = file(filename,'rb') # As it'll try to decode later on with utf-8, read it binary at this point.
        return self.parse(f.read())


class Point(object):
    '''
    Helper class representing a point.
    '''

    def __init__(self, *pts):
        self.x, self.y = pts

    def __getitem__(self, i):
        if i == 0:
            return self.x

        if i == 1:
            return self.y

        raise AssertionError('For 2d point can only get 0 or 1 (trying to get: %s)' % (i,))

    def __len__(self):
        return 2

    def __iter__(self):
        yield self.x
        yield self.y

    def distance(self, p2):
        x1, y1 = self
        x2, y2 = p2
        dist = math.sqrt((x2 - x1) ** 2 + (y2 - y1) ** 2)
        return dist

    def __str__(self):
        return '(%s, %s)' % (self.x, self.y)

    def __repr__(self):
        return 'Point(%s, %s)' % (self.x, self.y)


def angle_between_360_degrees(angle):
    while angle >= 360.0:
        angle -= 360.0
    while angle < 0.0:
        angle += 360.0
    if angle == 360.0:
        angle = 0.0

    return angle


def angle_to_x_axis_in_degrees(pt0, pt1):
    base_p0 = (0.0, 0.0)
    base_p1 = (1.0, 0.0)

    line1 = (pt0[0] - pt1[0], pt0[1] - pt1[1])
    line2 = (base_p1[0] - base_p0[0], base_p1[1] - base_p0[1])

    x1 = float(line1[0])
    y1 = float(line1[1])
    x2 = float(line2[0])
    y2 = float(line2[1])
    divide_by = (abs(x1 * x2) + abs(y1 * y2))

    if divide_by == 0.0:
        if pt0[1] > pt1[1]:
            return 90.0
        else:
            return 270.0
    else:
        tg = ((x1 * y2) - (x2 * y1)) / divide_by
        tan_degrees = math.degrees(math.atan(tg))

        if pt0[0] > pt1[0]:
            tan_degrees = -tan_degrees
        else:
            tan_degrees += 180.0

        angle = tan_degrees
        angle = angle_between_360_degrees(angle)

        return angle

def new_point_at_distance(pt, distance, angle):
    angle = float(angle)
    x, y = pt[0], pt[1]
    x += float(distance) * numpy.cos(numpy.deg2rad(angle))
    y += float(distance) * numpy.sin(numpy.deg2rad(angle))
    return float(x), float(y)




import nengo
import numpy as np

model = nengo.Network()
with model:
    a = nengo.Ensemble(n_neurons=100, dimensions=1)
    
    def stim_function(t):
        return np.sin(t*2*np.pi)
    stim = nengo.Node(stim_function)
    
    nengo.Connection(stim, a, synapse=None)
    
    output = nengo.Node(None, size_in=1)
    
    conn = nengo.Connection(a, output, function=lambda x:0,
                     learning_rule_type=nengo.PES(learning_rate=1e-4))

    def error_function(t, x):
        s = stim_function(t)
        diff = x - s
        return diff

    error = nengo.Node(error_function, size_in=1)
    nengo.Connection(output, error, synapse=None)
    nengo.Connection(error, conn.learning_rule, synapse=0)



# Question Answering with Control using SPA

# Now we will build this model again, using the spa (semantic pointer 
# architecture) package built into Nengo 2.0.

# Press the play button to run the simulation.
# The top graph is the input to the visual subnet. When this input is a 
# STATEMENT, there is no response shown in the motor graph and the input is 
# stored in memory (shown in the memory graph).  To see bound pairs (e.g.
# RED*CIRCLE) in memory, you need to right-click on the memory graph and
# select 'show pairs'.  This can be cluttered but is more informative.
# When the input to 'visual' is a QUESTION, the motor graph shows the 
# appropriate answer. For instance, when the input to visual is 
# QUESTION+BLUE (showin in the visual graphs), the output from motor is SQUARE.
# Note this simulation is fairly sensitive to the dimensionality, you may
# need multiple runs or a higher dimension to get the expected results.

# Setup the environment
import nengo
import nengo.spa as spa
from nengo.spa import Vocabulary
import numpy as np

D = 64  # the dimensionality of the vectors
rng = np.random.RandomState(15)
vocab = Vocabulary(dimensions=D, rng=rng, max_similarity=0.1)

#Adding semantic pointers to the vocabulary
CIRCLE=vocab.parse('CIRCLE')
BLUE=vocab.parse('BLUE')
RED=vocab.parse('RED')
SQUARE=vocab.parse('SQUARE')
ZERO=vocab.add('ZERO', [0]*D)

model = spa.SPA(label="Question Answering with Control", vocabs=[vocab])
with model:
    
    model.visual = spa.State(D)
    model.motor = spa.State(D)
    model.memory = spa.State(D, feedback=1, feedback_synapse=0.1)

    actions = spa.Actions(
        'dot(visual, STATEMENT) --> memory=visual',
        'dot(visual, QUESTION) --> motor = memory * ~visual'
        )
 
    model.bg = spa.BasalGanglia(actions)
    model.thalamus = spa.Thalamus(model.bg)

    #function for providing visual input
    def visual_input(t):
        if 0.1 < t < 0.3:
            return 'STATEMENT+RED*CIRCLE'
        elif 0.35 < t < 0.5:
            return 'STATEMENT+BLUE*SQUARE'
        elif 0.55 < t < 0.7:
            return 'QUESTION+BLUE'
        elif 0.75 < t < 0.9:
            return 'QUESTION+CIRCLE'
        else:
            return 'ZERO'
        
    
    #Inputs
    model.input = spa.Input(visual=visual_input)  



# Tutorial 24: Unbinding Concepts

# Now that we can combine information together into a single structure (see
# the previous tutorial), we also need to be able to extract information
# back out.  We do this by exploiting a pseudo-inverse property of the
# binding operation.  In particular, A*B*~B is approximately the same as A,
# where ~ means to take the pseudo-inverse of a vector by rearranging its
# elements (in particular, A*~B is the circular correlation of A and B).
#
# Using this approach, we can store bound information in memory and then
# query that memory to ask questions.  For this tutorial, we use shape and
# color as inputs exactly as in the previous tutorial.  Start by feeding in
# TRIANGLE and BLUE until the memory contains BLUE*TRIANGLE.  Now feed in
# CIRCLE and RED until the memory also contains RED*CIRCLE (this is the same
# as in the previous tutorial).  Now take away those inputs by setting the
# values of shape and color to nothing.
#
# Once the memory is set up in this way, we can query the memory by presenting
# inputs to the query population.  If you put in a query of RED you should
# get an answer of CIRCLE.  For CIRCLE you should get RED.  For BLUE you should
# get TRIANGLE.  And for TRIANGLE you should get BLUE.  Notice that the memory
# will gradually decay and fade the longer you try to run the system for.

import nengo
import nengo.spa as spa

D = 32  # the dimensionality of the vectors

model = spa.SPA()
with model:
    model.color = spa.State(D)
    model.shape = spa.State(D)
    model.memory = spa.State(D, feedback=1)
    model.query = spa.State(D)
    model.answer = spa.State(D)

    actions = spa.Actions(
        'memory = color * shape',
        'answer = memory * ~query',
        )
 
    model.cortical = spa.Cortical(actions)



import os

import nengo
import numpy as np

import nengo_gui


def test_lorenz():
    pyfile = os.path.join(
        nengo_gui.__path__[0], 'examples', 'tutorial', '15-lorenz.py')

    model_objects = {}
    nengo.utils.stdlib.execfile(pyfile, model_objects)

    with model_objects['model']:
        state_p = nengo.Probe(model_objects['x'], synapse=.01)

    with nengo.Simulator(model_objects['model']) as sim:
        sim.run(5)

    variance = np.var(sim.data[state_p], axis=0)
    assert np.all(variance > 40)



# Question Answering with Memory

# This model shows a form of question answering with memory. It binds two 
# features (color and shape) by circular convolution and stores them in a memory 
# population. It then provides a cue to the model at a later time to 
# determine the feature bound to that cue by deconvolution. This model exhibits 
# better cognitive ability since the answers to the questions are provided at a 
# later time and not at the same time as the questions themselves.

# **Note: A simplified method of building the model using the spa (semantic 
# pointer architecture) package in Nengo 2.0 is shown in the 
# spa_question-memory.py file in the same folder. 

# This model has parameters as described in the book, with memory population 
# having 1000 neurons over 20 dimensions. The memory population is capable of 
# storing a vector over time and it uses an integrator network to do so as 
# discussed in the book.

# The color input presented to the model is RED and then BLUE for 0.25 seconds 
# each before being turned off. In the same way the shape input is CIRCLE and 
# then SQUARE for 0.25 seconds each. Thus, when you run the model, it will start 
# by binding RED and CIRCLE for 0.25 seconds and then binding BLUE and SQUARE 
# for 0.25 seconds. The cue for deconvolving bound semantic pointers will be 
# turned off for 0.5 seconds and then starts cycling through CIRCLE, RED, 
# SQUARE, and BLUE within each second. The model will be able to determine the 
# correct answer using the cue, even when the color and shape inputs have been 
# turned off. 

# Press the play button to run the simulation.
# The graphs of shape, color, bound and memory show that first RED * CIRCLE and 
# then BLUE * SQUARE are convolved (bound) and loaded into the memory population, 
# so after 0.5 seconds the memory represents the superposition 
# RED * CIRCLE + BLUE * SQUARE. The last plot shows that the output is most 
# similar to the semantic pointer bound to the current cue. For example, when 
# the cue is CIRCLE, the output is most similar to RED.  Again, this is much
# easier to see in the spa_question-memory.py example.

#Setup the environment
import numpy as np
import nengo
from nengo.spa import Vocabulary
from nengo.spa import Memory

dim=20         # Number of dimensions 
N_input=300    # Number of neurons in population
N_conv=70      # Number of neurons per dimension in bind/unbind populations
N_mem=50       # Number of neurons per dimension in memory population

#Creating the vocabulary
rng = np.random.RandomState(7)  
vocab = Vocabulary(dimensions=dim, rng=rng, max_similarity=0.1)

model = nengo.Network(label='Question Answering with Memory', seed=12)  
with model:
    #Ensembles
    A = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='color')
    B = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='shape')
    C = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='cue')
    D = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='bound')
    E = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='output')
    
    #Creating memory population and connecting ensemble D to it
    tau = 0.4
    memory = nengo.networks.EnsembleArray(n_neurons=N_mem, n_ensembles=dim, 
                                            label='Memory')
    nengo.Connection(memory.output, memory.input, synapse=tau)     
    nengo.Connection(D, memory.input, synapse=0.01)
    
    #Creating the Bind network
    bind = nengo.networks.CircularConvolution(n_neurons=N_conv, dimensions=dim)
    nengo.Connection(A, bind.A)
    nengo.Connection(B, bind.B)
    nengo.Connection(bind.output, D) 
    
    #Creating the Unbind network
    unbind = nengo.networks.CircularConvolution(n_neurons=N_conv,  
                                        dimensions=dim, invert_a=True)
    nengo.Connection(C, unbind.A)
    nengo.Connection(memory.output, unbind.B)
    nengo.Connection(unbind.output, E)
    
    #Getting semantic pointer values
    CIRCLE=vocab.parse('CIRCLE').v
    BLUE=vocab.parse('BLUE').v
    RED=vocab.parse('RED').v
    SQUARE=vocab.parse('SQUARE').v
    ZERO=[0]*dim
    
    #function for providing color input
    def color_input(t):
        if t < 0.25:
            return RED
        elif t < 0.5:
            return BLUE
        else:
            return ZERO

    #function for providing shape input
    def shape_input(t):
        if t < 0.25:
            return CIRCLE
        elif t < 0.5:
            return SQUARE
        else:
            return ZERO

    #function for providing the cue
    def cue_input(t):
        if t < 0.5:
            return ZERO
        sequence = [ZERO, CIRCLE, RED, ZERO, SQUARE, BLUE]
        idx = int(((t - 0.5) // (1. / len(sequence))) % len(sequence))
        return sequence[idx]

    
    #Defining inputs
    inputA = nengo.Node(output=color_input, size_out=dim)
    inputB = nengo.Node(output=shape_input, size_out=dim)
    inputC = nengo.Node(output=cue_input, size_out=dim)
    
    #Connecting input to ensembles
    nengo.Connection(inputA, A)
    nengo.Connection(inputB, B)
    nengo.Connection(inputC, C)



# Tutorial 18: Networks

# To help organize larger models, you can make Networks inside of the main
# model Network.  
#
# In the graphic interface, the items inside these Networks are not shown
# by default.  If you double-click on a Network you can show (or hide) its
# internals.

# Nengo also comes with a collection of pre-made Networks that can help
# simplify the creation of large models.  For example, the code below uses a
# nengo.networks.EnsembleArray, which is a shortcut for creating a set of
# identical Ensembles.  It also provides a convenient "input" and "output"
# components that let you easily connect to or from all of the Ensembles at
# once.


import nengo

model = nengo.Network()
with model:
    stim_a = nengo.Node([0, 0, 0])
    stim_b = nengo.Node([0, 0])
    
    part1 = nengo.Network()
    with part1:
        a = nengo.Ensemble(n_neurons=100, dimensions=3)
        b = nengo.Ensemble(n_neurons=100, dimensions=2)
        c = nengo.Ensemble(n_neurons=300, dimensions=5)
        nengo.Connection(a, c[:3])
        nengo.Connection(b, c[3:])
    nengo.Connection(stim_a, a)
    nengo.Connection(stim_b, b)
    
    
    part2 = nengo.networks.EnsembleArray(n_neurons=50, n_ensembles=5)
    
    nengo.Connection(c, part2.input)
    


import numpy as np
import nengo
import nengo.spa as spa

try:
    from nengo_spa.examine import pairs
except ImportError:
    pairs = None

from nengo_gui.components.component import Component
from nengo_gui.components.spa_plot import SpaPlot


class SpaSimilarity(SpaPlot):
    """Line graph showing semantic pointer decoded values over time"""

    config_defaults = dict(max_value=1.5, min_value=-1.5,
                           show_pairs=False,
                           **Component.config_defaults)

    def __init__(self, obj, **kwargs):
        super(SpaSimilarity, self).__init__(obj, **kwargs)

        if isinstance(self.vocab_out, spa.Vocabulary):
            self.old_vocab_length = len(self.vocab_out.keys)
            self.labels = self.vocab_out.keys
        else:
            self.old_vocab_length = len(self.vocab_out)
            self.labels = tuple(self.vocab_out.keys())
        self.old_pairs_length = 0
        self.previous_pairs = False

        # Nengo objects for data collection
        self.node = None
        self.conn = None

    def add_nengo_objects(self, page):
        with page.model:
            if self.target.startswith('<'):
                output = getattr(self.obj, self.target[1:-1])
            else:
                output = self.obj.outputs[self.target][0]
            self.node = nengo.Node(self.gather_data,
                                   size_in=self.vocab_out.dimensions)
            self.conn = nengo.Connection(output, self.node, synapse=0.01)

    def remove_nengo_objects(self, page):
        """Undo the changes made by add_nengo_objects."""
        page.model.connections.remove(self.conn)
        page.model.nodes.remove(self.node)

    def gather_data(self, t, x):
        vocab = self.vocab_out

        if isinstance(vocab, spa.Vocabulary):
            length = len(vocab.keys)
        else:
            length = len(vocab)
        if self.old_vocab_length != length:
            self.update_legend(vocab)

        # get the similarity and send it
        key_similarity = np.dot(vocab.vectors, x)
        simi_list = ['{:.2f}'.format(simi) for simi in key_similarity]

        if self.config.show_pairs:

            # briefly there can be no pairs, so catch the error
            try:
                if isinstance(vocab, spa.Vocabulary):
                    pair_similarity = np.dot(vocab.vector_pairs, x)
                else:
                    pair_similarity = (np.dot(vocab.parse(p).v, x) for p in pairs(vocab))
                simi_list += ['{:.2f}'.format(simi) for simi in pair_similarity]
            except TypeError:
                pass

        if(simi_list != []):
            self.data.append(  '["data_msg", %g, %s]'
                             %( t, ",".join(simi_list) )  )

    def update_legend(self, vocab):
        # pass all the missing keys
        legend_update = []
        if isinstance(vocab, spa.Vocabulary):
            legend_update += (vocab.keys[self.old_vocab_length:])
            self.old_vocab_length = len(vocab.keys)
        else:
            legend_update += (list(vocab.keys())[self.old_vocab_length:])
            self.old_vocab_length = len(vocab)
        # and all the missing pairs if we're showing pairs
        if self.config.show_pairs:
            # briefly there can be no pairs, so catch the error
            try:
                key_pairs = list(pairs(vocab))
                legend_update += key_pairs[self.old_pairs_length:]
                self.old_pairs_length = len(key_pairs)
            except TypeError:
                pass

        self.data.append('["update_legend", "%s"]'
                         %('","'.join(legend_update)))

    def javascript(self):
        """Generate the javascript that will create the client-side object"""
        info = dict(uid=id(self), label=self.label, n_lines=len(self.labels),
                    synapse=0, pointer_labels=self.labels)
        json = self.javascript_config(info)
        return 'new Nengo.SpaSimilarity(main, sim, %s);' % json

    def message(self, msg):
        """Message receive function for show_pairs toggling and reset"""
        vocab = self.vocab_out
        # Send the new labels
        if self.config.show_pairs:
            vocab.include_pairs = True
            if isinstance(vocab, spa.Vocabulary):
                self.data.append(
                    '["reset_legend_and_data", "%s"]' % (
                        '","'.join(vocab.keys + vocab.key_pairs)))
                # if we're starting to show pairs, track pair length
                self.old_pairs_length = len(vocab.key_pairs)

            else:
                self.data.append(
                    '["reset_legend_and_data", "%s"]' % (
                        '","'.join(set(vocab.keys()) | pairs(vocab))))
                # if we're starting to show pairs, track pair length
                self.old_pairs_length = len(pairs(vocab))
        else:
            vocab.include_pairs = False
            if isinstance(vocab, spa.Vocabulary):
                self.data.append('["reset_legend_and_data", "%s"]'
                             % ('","'.join(vocab.keys)))
            else:
                self.data.append('["reset_legend_and_data", "%s"]'
                             % ('","'.join(vocab)))



# Addition

# In this model, you will see a transformation which is the basic property of 
# single neurons (i.e., addition). Addition transforms two inputs into a single 
# output which is their sum. You will construct a network that adds two inputs. 
# The network utilizes two communication channels going into the same neural 
# population. Addition is somewhat ‘free’, since the incoming currents from 
# different synaptic connections interact linearly.

# This model has ensembles A and B which represent the two inputs to be added. 
# The 'Sum' ensemble represents the added value. All the parameters used in the 
# model are as described in the book, with the sum ensemble having a radius of 
# 2 to account for the maximum range of summing the input values.

# While connecting the inputs to the ensembles A and B, the transform is 
# set to 1 (which is the default value) since this should be a communication 
# channel. However as described in the book, you can scale a represented 
# variable by a constant value by changing the transform. Example: if you  
# set the transform of ensemble B to 0 and ensemble A to 2 
# (i.e., nengo.Connection(input_A, A, transform=[2]) ), the sum will be twice 
# of the input_A. You will also need to set an appropriate radius for the 
# Sum ensemble to avoid saturation when you change the transform values. 

# Press the play button to run the simulation.
# The input_A and input_B graphs show the inputs to ensembles A and B 
# respectively. The graphs A and B show the decoded value of the activity of 
# ensembles A and B respectively. The sum graph shows that the decoded value of 
# the activity in the Sum ensemble provides a good estimate of the sum of inputs
# A and B. You can use the sliders to change the input values provided by the 
# input_A and input_B nodes.


#Setup the environment
import nengo
from nengo.dists import Uniform
from nengo.utils.functions import piecewise

#Create the network
model = nengo.Network(label='Scalar Addition')

with model:
    #Inputs to drive the activity in ensembles A and B 
    input_A = nengo.Node(piecewise({0: -0.75, 1.25: 0.5, 2.5: 0.70, 3.75: 0}))
    input_B = nengo.Node(piecewise({0: 0.25, 1.25: -0.5, 2.5: 0.85, 3.75: 0}))

    #Ensembles with 100 LIF neurons each
    # Represents the first input
    A = nengo.Ensemble(100, dimensions=1, max_rates=Uniform(100, 200))             
    # Represents the second input
    B = nengo.Ensemble(100, dimensions=1, max_rates=Uniform(100, 200))   
    # Reprsents the sum of two inputs
    Sum = nengo.Ensemble(100, dimensions=1, max_rates=Uniform(100, 200), 
                                    radius=2) 
        
    #Connecting the input nodes to ensembles
    nengo.Connection(input_A, A)
    nengo.Connection(input_B, B)
    
    #Connecting ensembles A and B to the Sum ensemble
    nengo.Connection(A, Sum)
    nengo.Connection(B, Sum)



'''
Tests for the NetGraph class.
'''

import numpy as np

import nengo
from nengo_gui.components.netgraph import NetGraph


def test_get_pre_post_obj():
    model = nengo.Network()
    with model:
        a = nengo.Ensemble(1, 10)
        b = nengo.Ensemble(1, 10)
        conn = nengo.Connection(a, b)
    assert a == NetGraph.connection_pre_obj(conn)
    assert b == NetGraph.connection_post_obj(conn)
    assert "normal" == NetGraph.connection_kind(conn)


def test_get_pre_post_obj_neurons():
    model = nengo.Network()
    with model:
        a = nengo.Ensemble(1, 10)
        b = nengo.Ensemble(1, 10)
        conn = nengo.Connection(a.neurons, b.neurons)
    assert a == NetGraph.connection_pre_obj(conn)
    assert b == NetGraph.connection_post_obj(conn)
    assert "normal" == NetGraph.connection_kind(conn)


def test_get_pre_post_obj_learning_rule():
    model = nengo.Network()
    with model:
        a = nengo.Ensemble(1, 10)
        b = nengo.Ensemble(1, 10)
        c = nengo.Ensemble(1, 10)
        conn_1 = nengo.Connection(a, b)
        conn_1.learning_rule_type = nengo.PES()
        conn_2 = nengo.Connection(c, conn_1.learning_rule)
    assert a == NetGraph.connection_pre_obj(conn_1)
    assert b == NetGraph.connection_post_obj(conn_1)
    assert "normal" == NetGraph.connection_kind(conn_1)

    assert c == NetGraph.connection_pre_obj(conn_2)
    assert b == NetGraph.connection_post_obj(conn_2)
    assert "modulatory" == NetGraph.connection_kind(conn_2)


def test_connection_inhibitory():
    model = nengo.Network()
    with model:
        a = nengo.Ensemble(1, 10)
        b = nengo.Ensemble(1, 10)
        conn_1 = nengo.Connection(a, b.neurons, transform=1 * np.ones((1, 10)))
        conn_2 = nengo.Connection(
            a, b.neurons, transform=-1 * np.ones((1, 10)))
        conn_3 = nengo.Connection(
            a,
            b.neurons,
            transform=np.array((0, -1, 0, 0, 0, 0, 0, 0, 0, 0)).reshape((1,
                                                                         10)))
        conn_4 = nengo.Connection(
            a, b.neurons, transform=-1 * np.zeros((1, 10)))
    assert "normal" == NetGraph.connection_kind(conn_1)
    assert "inhibitory" == NetGraph.connection_kind(conn_2)
    assert "inhibitory" == NetGraph.connection_kind(conn_3)
    assert "normal" == NetGraph.connection_kind(conn_4)




import warnings

import nengo_gui.gui


# dummy fn so that old scripts still work, but get a deprecation message
def Viz(*args, **kwargs):
    warnings.warn("nengo_gui.Viz is deprecated. Use nengo_gui.GUI instead.")
    return nengo_gui.GUI(*args, **kwargs)



# Structured Representations

# This model shows a method for constructing structured representations 
# using semantic pointers (high-dimensional neural vector representations). It 
# uses a convolution network to bind two Semantic Pointers and a Sum 
# network to cojoin two semantic pointers.

# **Note: This model can be simplified if built using the spa (semantic 
# pointer architecture) package in Nengo 2.0. This method is shown in the 
# spa_structure.py file in the same folder. 

# This model has parameters as described in the book, with the ensembles 
# having 20 dimensions and 300 neurons each. You will use the 
# 'nengo.networks.CircularConvolution' class in Nengo 2.0 to compute the 
# convolution (or binding) of two semantic pointers A and B.

# Since the collection of named vectors in a space forms a kind of "vocabulary" 
# as described in the book, you will create a vocabulary to build structured 
# representations out of it.

# Press the play button to run the simulation.
# The value graphs show the value of individual components of their respective 
# ensembles. They show the same information as the "value" graphs in the 
# Interactive Plots in Nengo 1.4 GUI as described in the book.

# Sum looks similar to B and A, whereas C looks very dissimilar because it
# is the result of a circular convolution.

#Setup the environment
import numpy as np
import nengo
from nengo.spa import Vocabulary

dim = 20              # Number of dimensions 
num_neurons = 300     # Number of neurons in each ensemble

#Creating a vocabulary
rng = np.random.RandomState(0)
vocab = Vocabulary(dimensions=dim, rng=rng)

model = nengo.Network(label = 'Structured Representation')
with model:
    #Input - Get the raw vectors for the pointers using `vocab['A'].v`
    inputA = nengo.Node(output=vocab['A'].v)
    inputB = nengo.Node(output=vocab['B'].v)
    
    #Ensembles with 300 neurons and 20 dimensions
    A = nengo.Ensemble(num_neurons, dimensions=dim)     # Represents inputA
    B = nengo.Ensemble(num_neurons, dimensions=dim)     # Represents inputB
    C = nengo.Ensemble(num_neurons, dimensions=dim)     # Represents the
                                                        # convolution of A and B 
    Sum =  nengo.Ensemble(num_neurons, dimensions=dim)  # Represents the sum of 
                                                        # A and B 
    
    #Creating the circular convolution network with 70 neurons per dimension
    Bind = nengo.networks.CircularConvolution(70, dimensions=dim)
    
    #Connecting the input to ensembles A and B
    nengo.Connection(inputA, A)
    nengo.Connection(inputB, B)  
    
    #Projecting ensembles A and B to the Bind network
    nengo.Connection(A, Bind.A)
    nengo.Connection(B, Bind.B)
    nengo.Connection(Bind.output, C)
    
    #Projecting ensembles A and B to the Sum ensemble
    nengo.Connection(A, Sum)
    nengo.Connection(B, Sum) 



# Tutorial 7: Multiple Dimensions

# Ensembles of neurons do not just have to represent one thing.  Instead,
# a group of neurons can represent multiple values at the same time.  We call
# the number of values represented at once the "dimensions" of the Ensemble.
# So, if a group of neurons is supposed to represent the spatial location of
# something in three dimensions (x, y, z), then we say that it has dimensions=3.

# In this case, three different values are being decoded from each of the
# groups of neurons.  Nengo decodes these values by finding different ways
# of weighting together the actual spiking output (top graphs) in order to
# produce the bottom graphs.  

# Nengo Tip: If you change the number of dimensions, the graphs and sliders
# may not update to reflect those changes.  You can force them to do so by
# removing them and re-creating them.  To remove them, right-click on the graph
# or slider and select "Remove".  To re-create them, right-click on the item
# you want to make a graph for and select an option.  "Slider" makes the
# sliders for controlling input; "Value" shows the decoded value being
# represented; and "Spikes" shows the individual spiking activity of the
# neurons.

import nengo

model = nengo.Network()
with model:
    stim = nengo.Node([0, 0, 0])
    a = nengo.Ensemble(n_neurons=200, dimensions=3)
    nengo.Connection(stim, a)
    
    b = nengo.Ensemble(n_neurons=200, dimensions=3)
    nengo.Connection(a, b)


import argparse
import errno
import logging
import os.path
import threading
import webbrowser

import nengo_gui
import nengo_gui.gui
from nengo_gui.guibackend import ModelContext, GuiServerSettings
from nengo_gui.password import gensalt, hashpw, prompt_pw


def old_main():
    print("'nengo_gui' has been renamed to 'nengo'.")
    print("Please run 'nengo' in the future to avoid this message!\n")
    main()


def main():
    try:
        browser_help = ('browser to use (options: "%s")' % 
                        '", "'.join(webbrowser._tryorder))
    except (AttributeError, TypeError):
        # just in case the undocumented webbrowser._tryorder changes
        browser_help='browser to use (e.g. chrome, firefox)'

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-p', '--password', dest='password', metavar='PASS',
        nargs='?', default=False, const=True, type=str,
        help='password for remote access')
    parser.add_argument(
        '-l', '--listen', nargs=1, default=['localhost'], type=str,
        help="Network interface to listen on for incoming connections. Set to "
        "'*' to allow connections on all network interfaces.")
    parser.add_argument(
        '--cert', nargs=1, default=[None], type=str,
        help="SSL certificate file")
    parser.add_argument(
        '--key', nargs=1, default=[None], type=str, help="SSL key file")
    parser.add_argument(
        '--unsecure', action='store_true', help='do not use SSL security')
    parser.add_argument(
        '-P', '--port', dest='port', metavar='PORT',
        type=int, help='port to run server on')
    parser.add_argument(
        'filename', nargs='?', type=str, help='initial file to load')
    parser.add_argument(
        '--debug', action='store_true', help='turn on debug logging')
    parser.add_argument(
        '-b', '--backend', metavar='BACKEND',
        default='nengo', type=str, help='default backend to use')
    parser.add_argument('--browser', dest='browser', type=str,
        metavar='BROWSER', default=True,
        help=browser_help)
    parser.add_argument('--no-browser', dest='browser', action='store_false')
    parser.add_argument(
        '--auto-shutdown', nargs=1, type=float,
        help="Time limit before automatic shutdown. Set to 0 to deactivate.",
        default=[2])
    parser.set_defaults(browser=True)
    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig()

    if args.port is None:
        port = 8080
    else:
        port = args.port

    if args.password:
        if args.password is True:
            password_hash = hashpw(prompt_pw(), gensalt())
        else:
            password_hash = hashpw(args.password, gensalt())
    else:
        password_hash = None

    host = args.listen[0]
    if host == '*':
        host = ''
    server_settings = GuiServerSettings(
        (host, port), args.auto_shutdown[0], password_hash=password_hash,
        ssl_cert=args.cert[0], ssl_key=args.key[0])
    if (host != 'localhost' and not server_settings.use_ssl and
            not args.unsecure):
        raise ValueError(
            "Listening on external network interfaces only allowed with SSL.")

    try:
        if args.filename is None:
            filename = os.path.join(
                nengo_gui.__path__[0], 'examples', 'default.py')
        else:
            filename = args.filename
        page_settings = nengo_gui.page.PageSettings(backend=args.backend)
        s = None
        while s is None:
            try:
                s = nengo_gui.gui.InteractiveGUI(
                    ModelContext(filename=filename), server_settings,
                    page_settings=page_settings)
            except EnvironmentError as err:
                if args.port is None and err.errno == errno.EADDRINUSE:
                    port += 1
                    if port > 0xFFFF:
                        raise
                    server_settings.listen_addr = (host, port)
                else:
                    raise

        s.server.auto_shutdown = args.auto_shutdown[0]

        if args.browser:
            protocol = 'https:' if server_settings.use_ssl else 'http:'
            host = 'localhost'
            port = s.server.server_port
            if args.browser is True:
                wb = webbrowser.get()
            else:
                try:
                    wb = webbrowser.get(args.browser)
                except webbrowser.Error:
                    try:
                        print('Known browsers: \n  %s' % 
                              '\n  '.join(webbrowser._tryorder))
                    except (AttributeError, TypeError):
                        # just in case the undocumented webbrowser._tryorder
                        #  changes
                        print('Could not determine the list of known browsers.')
                    raise
            t = threading.Thread(
                target=wb.open,
                args=('%s//%s:%d/?token=%s' % (
                    protocol, host, port, s.server.gen_one_time_token()),))
            t.start()

        s.start()
    finally:
        logging.shutdown()


if __name__ == '__main__':
    main()



# Tutorial 6: Adding

# If we make two Connections into the same Ensemble, the Ensemble will get
# input from both sources, and it will end up representing the sum of all of
# its inputs.  Here, we use this to add two values together.

# Notice that the value being represented by Ensemble c is, most of the time,
# the sum of the two inputs (a and b).  However, if that value gets too large
# (or too small), it does not work very well.  This is because every Ensemble
# has a particular range of values that it expects to work with.  We call this
# the "radius" of the Ensemble, and the default radius is 1, which means that
# the Ensemble is best at representing values between -1 and 1.  You can
# change this radius by adjusting the value specified below.  Try changing it
# so that it says "radius=2".  Now the neurons will be good at representing
# -2 to 2.  However, in order to see that on the graph for Ensemble c, you
# will have to adjust the range it is displaying.  Right-click on the graph,
# select "Set range" and set it to "-2,2".  Now it should add correctly over
# the full range of inputs.


import nengo

model = nengo.Network()
with model:
    stim_a = nengo.Node(0)
    a = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim_a, a)
    
    stim_b = nengo.Node(0)
    b = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim_b, b)
    
    c = nengo.Ensemble(n_neurons=50, dimensions=1, radius=1)
    nengo.Connection(a, c)
    nengo.Connection(b, c)
    



# Tutorial 4: Connecting neurons

# So far, we have just fed input to a group of neurons and read their output.
# In order to do more interesting things, we need to be able to connect one
# group of neurons to another.  In this example, we have two groups of neurons
# connected together.

# If you just make a Connection between two groups of neurons, Nengo will
# find the connection weights between each of the neurons such that whatever
# value is being represented by the first group of neurons will be passed on
# to the second group of neurons.  Notice that now if you move the slider,
# this affects the value in the first group of neurons, and these in turn
# affect the value in the second group of neurons.

# Whenever you make a connection, you can specify a "synapse" value.  This
# indicates the properties of the neurotransmitters and synapses (the actual
# connections between neurons).  The most important paramter is the
# time constant: the amount of time it takes for the effects of a single spike
# to wear off.  For some parts of the brain, this is very fast (0.002 seconds)
# and in other parts it is very slow (0.2 seconds).  If you don't specify it,
# Nengo will assume you want a fairly fast connection (0.005 seconds).  Try
# changing the synapse value between "a" and "b" to be much slower (0.2). This
# should make the value stored in b change more slowly.

import nengo

model = nengo.Network()
with model:
    stim = nengo.Node(0)
    a = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim, a)
    
    b = nengo.Ensemble(n_neurons=50, dimensions=1)
    
    nengo.Connection(a, b, synapse=0.01)



# Tutorial 17: Neuron Models

# Nengo supports multiple different types of neurons.  The default is the
# "Leaky Integrate-and-Fire" or LIF neuron.  Other supported ones are shown
# here.  The LIFRate neuron acts like the LIF neuron, but does not have spikes.
# The Sigmoid is a standard model used in machine learning.  The
# RectifiedLinear model simply outputs the absolute value of its input (the
# simplest possible neuron-like operation).  The most complex neuron model
# here is the Izhikevich neuron, which has four parameters to adjust and
# has been shown to map very closely to a large number of real biological
# neurons.

import nengo

model = nengo.Network()
with model:
    
    stim = nengo.Node(0)
    
    a = nengo.Ensemble(n_neurons=50, dimensions=1,
                       neuron_type=nengo.LIF(tau_rc=0.02, tau_ref=0.002))
                       
    b = nengo.Ensemble(n_neurons=50, dimensions=1,
                       neuron_type=nengo.LIFRate(tau_rc=0.02, tau_ref=0.002))
                       
    c = nengo.Ensemble(n_neurons=50, dimensions=1,
                       neuron_type=nengo.Sigmoid(tau_ref=0.002),
                       max_rates=nengo.dists.Uniform(250., 400.))

    d = nengo.Ensemble(n_neurons=50, dimensions=1,
                       neuron_type=nengo.RectifiedLinear())

    e = nengo.Ensemble(n_neurons=50, dimensions=1,
                       neuron_type=nengo.Izhikevich(
                           tau_recovery=0.02,
                           coupling=0.2,
                           reset_voltage=-65,
                           reset_recovery=8))

    nengo.Connection(stim, a)
    nengo.Connection(stim, b)
    nengo.Connection(stim, c)
    nengo.Connection(stim, d)
    nengo.Connection(stim, e)    


import numpy as np

import nengo
from nengo.utils.ensemble import response_curves, tuning_curves


class PlotInfo(object):
    def __init__(self, title, plot="none"):
        self.title = title
        self.plot = plot
        self.warnings = []
        self.x = None
        self.y = None
        self.x_label = None
        self.y_label = None

    def to_dict(self):
        x, y = self.x, self.y
        if self.plot == "multiline":
            assert self.x.shape[0] == self.y.shape[1]
            x = self.x.tolist()
            y = [yy.tolist() for yy in self.y]
        return {
            'plot': self.plot,
            'title': self.title,
            'warnings': self.warnings,
            'x': x,
            'y': y,
            'x_label': self.x_label if self.x_label != None else "",
            'y_label': self.y_label if self.y_label != None else "",
        }

def response_curve_plot(ens, sim):
    rc = PlotInfo("Response curves", plot="multiline")
    rc.x, rc.y = response_curves(ens, sim)
    rc.x_label = "Input current"
    rc.y = rc.y.T
    rc.y_label = "Firing rate (Hz)"

    if len(rc.y.shape) == 1:
        rc.y.shape = 1, rc.y.shape[0]
    if ens.n_neurons > 200:
        rc.warnings.append("Only showing the first 200 neurons.")
        rc.y = rc.y[:200]
    return rc.to_dict()

def tuning_curve_plot(ens, sim):
    tc = PlotInfo("Tuning curves")

    if ens.dimensions == 1:
        tc.plot = "multiline"
        tc.x, tc.y = tuning_curves(ens, sim)
        tc.x_label = "x"
        tc.y = tc.y.T
        tc.y_label = "Firing rate (Hz)"

        if ens.n_neurons > 200:
            tc.warnings.append("Only showing the first 200 neurons.")
            tc.y = tc.y[:200]
    else:
        tc.warnings.append("Tuning curves only shown for "
                           "one-dimensional ensembles.")
    return tc.to_dict()

def node_output_plot(node):
    f_out = PlotInfo("Node output")
    if node.size_in > 0:
        f_out.warnings.append("Node output only shown when 'size_in' is 0.")
    else:
        f_out.plot = "multiline"
        if callable(node.output):
            dt = 0.001
            f_out.x = np.arange(dt, 1.0, dt)
            f_out.y = np.asarray([node.output(x) for x in f_out.x])
        else:
            # Don't bother with all the copies if it's static
            f_out.x = np.asarray([0, 1.0])
            f_out.y = np.hstack((node.output, node.output))
        if f_out.y.ndim == 1:
            f_out.y = f_out.y[:, np.newaxis]
        f_out.y = f_out.y.T
    return f_out.to_dict()



import json

try:
    from nengo.utils.compat import escape
except ImportError:
    import sys
    PY2 = sys.version_info[0] == 2
    if PY2:
        from cgi import escape as cgi_escape
        escape = lambda s, quote=True: cgi_escape(s, quote=quote)
    else:
        from html import escape

from nengo.utils.progress import ProgressBar, timestamp2timedelta

from nengo_gui.components.component import Component


class Progress(Component, ProgressBar):
    def __init__(self):
        super(Progress, self).__init__()
        self.progress = None

    def update(self, progress):
        self.progress = progress

    def update_client(self, client):
        if self.progress is not None:
            client.write_text(json.dumps({
                'name_during': escape(getattr(self.progress, 'name_during', 'Building')),
                'name_after': escape(getattr(self.progress, 'name_after', 'Build')),
                'progress': self.progress.progress,
                'max_steps': self.progress.max_steps,
                'elapsed_time': str(timestamp2timedelta(
                    self.progress.elapsed_seconds())),
                'eta': str(timestamp2timedelta(self.progress.eta())),
                'finished': self.progress.finished,
                'success': self.progress.success
            }))
            if self.progress.finished:
                self.progress = None

    def javascript(self):
        info = dict(uid=id(self))
        js = self.javascript_config(info)
        return 'progress = new Nengo.Progress({});\n'.format(js)



# Representing a Vector

# In nengo, ensembles of neurons represent information. In this model, 
# you will represent a two-dimensional vector with a single ensemble of 
# LIF (leaky integrate-and-fire) neurons. Though this is a case of 
# two-dimensional vector representation, but the ideas naturally  
# generalize to any number of dimensions.

# This model has paramenters as described in the book, with the neurons 
# in the ensemble having two dimensions. Since the default max rates in 
# Nengo 2.0 are (200, 400), so you have to explicitly specify them to be 
# (100, 200) to create the same model as in the book. The default values 
# of tauRC, tauRef, intercepts, radius and expected noise in Nengo 2.0 
# are same as those mentioned in the book.

# Press the play button to run the simulation.
# The cos and sin graphs show the two dimensional input provided to the 
# ensemble and the top right graph shows the the decoded estimate of this 
# two dimensional input. The graph on bottom right shows the XY-value i.e., 
# the state represented by one dimension of the ensemble vs the state 
# represented by the other dimension.

# Grab and move the sliders to manually control the input to the neurons.

#Setup the envirnment
import numpy as np
import nengo
from nengo.dists import Uniform

model = nengo.Network(label='2D Representation')
with model:
    #Input Nodes
    sin = nengo.Node(output=np.sin)
    cos = nengo.Node(output=np.cos)
    
    #Ensemble with 100 LIF neurons which represents a 2-dimensional signal
    x = nengo.Ensemble(100, dimensions=2, max_rates=Uniform(100, 200))
    
    #Get the neuron encoders
    encoders = x.encoders.sample(100,2)
    
    #Connecnting input to ensemble
    #The indices in ensemble 'x' define which dimension the input will project 
    #to
    nengo.Connection(sin, x[0])
    nengo.Connection(cos, x[1])



import warnings

try:
    from jedi.api import Script
except ImportError:
    warnings.warn(
        "Install the jedi module to get autocompletion in Nengo GUI.")

    class Script(object):
        def __init__(*args, **kwargs):
            pass

        def completions(self):
            return []


def get_completions(code, line, column, path=None):
    script = Script(code, line, column, path=path)
    return script.completions()



# Tutorial 19: Semantic Pointers

# If we want to represent conceptual information, we need a way to represent
# concepts and symbol-like manipulations using Nengo.  We do this by treating
# concepts like vectors: high-dimensional numerical data.  That is, each
# concept (like DOG or CAT or VERB or RUNNING) corresponds to a particular
# vector.  So HOUSE might be [0.2, 0.6, -0.7, -0.2, 0] and DREAM might be
# [-0.1, -0.7, 0, 0.9, 0.2].
#
# We call these "semantic" because, in general, we would choose these
# numerical values such that concepts with similar semantics (like DOG and
# CAT) might have similar numerical values.

# To help work with these vectors, we introduce a new collection of pre-built
# Networks, and a new type of graph.  The new pre-built Networks can be
# accessed via nengo.spa ("spa" stands for "Semantic Pointer Architecture",
# a collection of tools for making these sorts of models.  A spa.State is
# a Network that holds one vector representing a semantic pointer.
# If the feedback parameter is set, then the spa.State acts like a memory,
# remembering the vector even after the input has been taken away.
#
# The new graph is a "Semantic pointer" graph.  Rather than showing the
# individual values, it shows how close the currently represented vector is
# to the ideal original vectors.  Furthermore, you can use it as an input
# system as well, and define new concepts.

# Press play to start the simulation running.  Now right-click on the "vision"
# graph (the blank space above the "vision" box in the diagram).  Select "Set
# value..." and put in CAT as a value.  Nengo will randomly generate a new
# vector to mean CAT and will feed it into the model.  Since the vision system
# is connected to the memory system, the memory will gradually store CAT as
# well.  Now right-click on vision, go to "Set value" and just leave it blank
# (click OK).  The CAT in vision will go away, but the memory should continue
# to store CAT.  Now set the vision to DOG.  This new vector will be fed into
# the memory, gradually replacing the CAT.  (The size and darkness of the words
# indicates how similar the vector is to the ideal vectors for CAT and DOG).

import nengo
import nengo.spa as spa

D = 32  # the dimensionality of the vectors

model = spa.SPA()
with model:
    model.vision = spa.State(D)

    model.memory = spa.State(D, feedback=1)

    nengo.Connection(model.vision.output, model.memory.input,
                     transform=0.1)



# Sequencing

# This model uses the basal ganglia model to cycle through a sequence of five 
# states (i.e., A->B->C->D->E->A->...). The model incorporates a 
# working memory component (state), which allows the basal ganglia to update 
# that memory based on a set of condition/action mappings.

# The model has parameters as described in the book. In Nengo 1.4 (and the book) 
# separate 'Rules' and 'Sequence' classes were created. However, this is not 
# needed in Nengo 2.0 since you can directly specify the rules using the 
# built-in 'Actions' class in the spa (semantic pointer architecutre) package. This 
# class takes a string definition of the action as an input as shown in the code 
# where '-->' is used to split the action into condition and effect, otherwise 
# it is treated as having no condition and just effect.

# The syntax for creating an input function in Nengo 2.0 is also different from 
# that in Nengo 1.4 mentioned in the book. The syntax for Nengo 2.0 which is 
# used here is spa.input(module=function). The first parameter 'module' 
# refers to name of the module that you want to provide input to and the second 
# parameter 'function' refers to the function to execute to generate inupts to 
# that module. The functions should always return strings, which will then be 
# parsed by the relevant vocabulary.

# In Nengo 1.4, a memory element for representing the state was created by using 
# Buffer() as described in the book. However, in Nengo 2.0, we use 
# State() with feedback parameter set to 1 for creating a memory module capable 
# of storing a vector over time.

# Press the play button in the visualizer to run the simulation. 
# The graph on the top-left shows the semantic pointer representation of the 
# values stored in the state ensemble. The graph on the bottom-right shows the 
# current transition or the action being executed, and the graph on the 
# top-right shows the utility (similarity) of the current basal ganglia input 
# (i.e., state) with the possible vocabulary vectors.

# The book describes that the results of the model can be seen through the 
# visualizer in Nengo 1.4 GUI which has a "Utility" box and the "Rules" box. 
# Note that the bottom-right graph shows the same information as seen in the 
# "Rules" box and top-right graph shows the same information as seen in the 
# "Utility" box.

#Setup the environment
import nengo                
from nengo import spa       #import spa related packages

#Number of dimensions for the Semantic Pointers
dimensions = 16

#Make a model with the SPA network
model = spa.SPA(label='Sequence')
with model:
    #Creating a working memory/cortical element 
    model.state = spa.State(dimensions=dimensions, feedback=1, 
                                feedback_synapse=0.01)
    
    #Specifying the action mappings (rules) for BG and Thal 
    actions = spa.Actions(
        'dot(state, A) --> state = B',
        'dot(state, B) --> state = C',
        'dot(state, C) --> state = D',
        'dot(state, D) --> state = E',
        'dot(state, E) --> state = A'
    )
    
    #Creating the BG and Thalamus components that confirm to the specified rules
    model.BG = spa.BasalGanglia(actions=actions)    
    model.thal = spa.Thalamus(model.BG)             
    
    #Function that provides the model with an initial input semantic pointer.
    def start(t):
        if t < 0.1:          #Duration of the initial input = 0.1
            return 'D'
        else:
            return '0'
        
    #Input
    model.input = spa.Input(state=start)


# Tutorial 5: Computing a function

# Whenever we make a Connection between groups of neurons, we don't have to
# just pass the information from one group of neurons to the next.  Instead,
# we can also modify that information.  We do this by specifying a function,
# and Nengo will connect the individual neurons to best approximate that
# function.

# In the example here, we are computing the square of the value.  So for an
# input of -1 it should output 1, for 0 it should output 0, and for 1 it should
# output 1.

# You can change the function by adjusting the computations done in the
# part of the code labelled "compute_this".  This can be any arbitrary Python
# function.  For example, try computing the negative of x ("return -x").  Try
# the absolute value ("return abs(x)").  You can also try more complex
# functions like "return 1 if x > 0 else 0".

import nengo

model = nengo.Network()
with model:
    stim = nengo.Node(0)
    a = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim, a)
    
    b = nengo.Ensemble(n_neurons=50, dimensions=1)
    
    def compute_this(x):
        return x * x
    
    nengo.Connection(a, b, synapse=0.01, function=compute_this)



# Tutorial 10: Transforms and scaling

# When making Connections, we may want to scale the values being represented.
# For example, we might want to just multiply a value by a fixed number like
# 0.1 or 10 or something like that.  Since this value doesn't change, we do
# not need a full multiplication system like in the previous tutorial.

# We could implement this sort of connection by writing a function.  However,
# this is such a common thing to do that Nengo has a shortcut for this by
# having a "transform" parameter.  The examples below show the equivalence of
# these two approaches.  In particular, b1 and b2 both take the value from a
# and multiply it by -0.5.  For a multidimensional example, d1 and d2 both take
# values from c and compute 2*c[0]-c[1]-c[2], but do so in different ways.  In
# both cases the resulting models are identical.

# You can use this trick to quickly define any linear transformation on the
# values represented by the Ensembles.

import nengo

model = nengo.Network()
with model:
    stim_a = nengo.Node(0)
    a = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim_a, a)
    
    b1 = nengo.Ensemble(n_neurons=50, dimensions=1)
    b2 = nengo.Ensemble(n_neurons=50, dimensions=1)
    
    # the long way to do it
    def simple_function(a):
        return -0.5 * a
    nengo.Connection(a, b1, function=simple_function)
    # the shortcut way to do it
    nengo.Connection(a, b2, transform=-0.5)
    
    
    stim_c = nengo.Node([0, 0, 0])
    c = nengo.Ensemble(n_neurons=200, dimensions=3)
    nengo.Connection(stim_c, c)
    
    
    d1 = nengo.Ensemble(n_neurons=50, dimensions=1)
    d2 = nengo.Ensemble(n_neurons=50, dimensions=1)
    
    # the long way to do it
    def harder_function(c):
        return 2 * c[0] - c[1] - c[2]
    nengo.Connection(c, d1, function=harder_function)
    # the shortcut way to do it
    nengo.Connection(c, d2, transform=[[2, -1, -1]])



from __future__ import print_function

import atexit
import socket
import threading
import time
import uuid
import warnings
import weakref
try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen

from IPython import get_ipython
from IPython.display import display, HTML

import nengo_gui


class ConfigReuseWarning(UserWarning):
    pass


class IPythonViz(object):
    servers = weakref.WeakValueDictionary()
    threads = weakref.WeakValueDictionary()
    configs = set()

    host = 'localhost'

    def __init__(self, model, cfg=None, height=600):
        self.height = height

        self._started = False

        if cfg is None:
            cfg = get_ipython().mktempfile()

        self._server_thread, server = self.start_server(cfg, model)
        self.port = server.server.server_port

        self.url = self.get_url(
            self.host, self.port, token=server.server.auth_token)

    @staticmethod
    def get_url(host, port, action=None, token=None):
        url = 'http://{host}:{port}/'.format(host=host, port=port)
        if action is not None:
            url += action
        if token is not None:
            url += '?token=' + token
        return url

    @classmethod
    def start_server(cls, cfg, model):
        # Make sure only one server is writing the same config.
        server_thread = cls.threads.get(cfg, None)
        server = cls.servers.get(cfg, None)
        existent = server_thread is not None and server is not None
        if existent and server_thread.is_alive():
            warnings.warn(ConfigReuseWarning(
                "Reusing config. Only the most recent visualization will "
                "update the config."))
            for page in server.server.pages:
                page.save_config(force=True)
                page.filename_cfg = get_ipython().mktempfile()
                cls.servers[page.filename_cfg] = server
                cls.threads[page.filename_cfg] = server_thread

        name = model.label
        server_settings = nengo_gui.guibackend.GuiServerSettings(
            ('localhost', 0))
        model_context = nengo_gui.guibackend.ModelContext(
            model=model, locals=get_ipython().user_ns, filename=None,
            writeable=False)
        page_settings = nengo_gui.page.PageSettings(
            filename_cfg=cfg,
            editor_class=nengo_gui.components.editor.NoEditor)
        server = nengo_gui.gui.BaseGUI(
            model_context, server_settings, page_settings)
        server_thread = threading.Thread(target=server.start)
        server_thread.start()
        cls.servers[cfg] = server
        cls.threads[cfg] = server_thread
        cls.configs.add(cfg)
        return server_thread, server

    def wait_for_startup(self):
        while self._server_thread.is_alive() and not self._started:
            try:
                s = socket.create_connection((self.host, self.port), 0.1)
                self._started = True
            except Exception:
                pass
            else:
                s.close()

    @classmethod
    def shutdown_all(cls, timeout=None):
        def get_timeout(start=time.clock()):
            if timeout is None:
                return None
            else:
                return max(0.001, timeout - (time.clock() - start))

        for cfg in cls.configs:
            server = cls.servers.get(cfg, None)
            server_thread = cls.threads.get(cfg, None)
            if server_thread is not None and server_thread.is_alive():
                if server is not None:
                    urlopen(
                        cls.get_url(cls.host, server.server_port, 'shutdown'),
                        timeout=get_timeout()).read()
                server_thread.join(get_timeout())

    def _ipython_display_(self):
        self.wait_for_startup()
        if self._server_thread.is_alive():
            display(HTML('''
                <div id="{id}">
                    <iframe
                        src="{url}"
                        width="100%"
                        height="{height}"
                        frameborder="0"
                        class="cell"
                        style="border: 1px solid #eee;"
                        allowfullscreen></iframe>
                </div>
            '''.format(url=self.url, id=uuid.uuid4(), height=self.height)))
        else:
            print("Server is not alive.")


atexit.register(IPythonViz.shutdown_all, timeout=5)



import os
import time

from nengo_gui import testing_tools as tt


def set_cloud_value(driver, spa_nets):
    for name in spa_nets:
        prompt_script = """
var items = Nengo.Component.components;
items.forEach(function(pointer) {
    if (pointer.label.innerHTML === '%s' &&
            pointer.constructor === Nengo.Pointer) {
        pointer.set_value();
    }
});
"""
        time.sleep(0.6)
        driver.execute_script(prompt_script % name)
        time.sleep(1.0)
        field = driver.find_element_by_xpath("//*[@id='singleInput']")
        time.sleep(0.6)
        field.send_keys(spa_nets[name])
        accept = driver.find_element_by_xpath("//*[@id='OK']")
        time.sleep(0.3)
        accept.click()


def test_spa(driver):
    """Tests the functionality of SPA simulations"""

    try:
        test_file = """
import nengo
import nengo.spa as spa

D = 2  # the dimensionality of the vectors

model = spa.SPA()
with model:
    model.color = spa.State(D)
    model.shape = spa.State(D)
    model.memory = spa.State(D, feedback=1)
    model.query = spa.State(D)
    model.answer = spa.State(D)

    actions = spa.Actions(
        "memory = color * shape",
        "answer = memory * ~query",
        )

    model.cortical = spa.Cortical(actions)
"""
        tt.reset_page(driver)
        time.sleep(0.2)
        tt.update_editor(driver, test_file)
        tt.mouse_scroll(driver, 200)

        # Generates semantic pointer clouds for each network
        driver.execute_script("""
var a = Nengo.netgraph.svg_objects;
for (model in a) {
    if (a[model].sp_targets.length > 0) {
        a[model].create_graph('Pointer', a[model].sp_targets[0]);
        a[model].create_graph('SpaSimilarity', a[model].sp_targets[0]);
    }
}
""")
        time.sleep(1)
        # Ensures the simulation has started

        hang_time = 100  # alloted time until test fails
        time_script = ("var time = $('#ticks_tr');\n"
                       "return time.find('td').text();\n")
        tt.start_stop_sim(driver)
        time_start = time.time()
        while time.time() - time_start < hang_time:
            sim_time = driver.execute_script(time_script)
            if(float(sim_time) > 0):
                break
            time.sleep(1)
        else:
            assert False, "Did not compile in %f seconds" % hang_time

        # Sets the semantic pointers appropriately
        spa_values = {"shape": "CIRCLE", "color": "BLUE", "query": "CIRCLE"}
        set_cloud_value(driver, spa_values)
        time.sleep(10)
        result = driver.execute_script("""
var objects = Nengo.Component.components;
var answer = objects.filter(function(item) {
    return item.label.innerHTML == "answer";
})[0];
var answer_data = answer.data_store.data[0];
var result = answer_data.pop();
return result;
""")

        data_script = """
return Nengo.Component.components.filter(function(item) {
    return (item.constructor === Nengo.SpaSimilarity);
})[0].data_store.data;
"""
        plot_data = driver.execute_script(data_script)

        # Checks that the 'answer' is BLUE
        assert any("BLUE" in x for x in result)

        # Checks the dimensionality of the spa similarity plot
        assert len(plot_data) == 2 and len(plot_data[0]) > 1

        set_cloud_value(driver, {"color": "ORANGE"})

        time.sleep(10)

        plot_data = driver.execute_script(data_script)

        # Checks that the data store grows when input changes
        assert(len(plot_data) == 3 and len(plot_data[0]) > 1)

    except:
        # Travis Only: On fail takes screenshot and uploads it to imgur
        if('TRAVIS' in os.environ):
            tt.imgur_screenshot(driver)
        raise



import nengo
import numpy as np

model = nengo.Network(seed=0)
with model:
    a = nengo.Ensemble(n_neurons=400, dimensions=2)
    
    def stim_func(t):
        return np.cos(t), np.sin(t)
    
    stim = nengo.Node(stim_func)
    
    nengo.Connection(stim, a, synapse=None)
    



import struct

import nengo
from nengo import spa

from nengo_gui.components.component import Component


class Value(Component):
    """The server-side system for a Value plot."""

    # the parameters to be stored in the .cfg file
    config_defaults = dict(max_value=1, min_value=-1,
                           show_legend=False, legend_labels=[],
                           synapse=0.01,
                           **Component.config_defaults)

    def __init__(self, obj):
        super(Value, self).__init__()
        # the object whose decoded value should be displayed
        self.obj = obj

        # the pending data to be sent to the client
        self.data = []

        # grab the output of the object
        self.output = obj
        default_out = Value.default_output(self.obj)
        if default_out is not None:
            self.output = default_out

        # the number of data values to send
        self.n_lines = int(self.output.size_out)

        # the binary data format to sent in.  In this case, it is a list of
        # floats, with the first float being the time stamp and the rest
        # being the vector values, one per dimension.
        self.struct = struct.Struct('<%df' % (1 + self.n_lines))

        # Nengo objects for data collection
        self.node = None
        self.conn = None

    def attach(self, page, config, uid):
        super(Value, self).attach(page, config, uid)
        # use the label of the object being plotted as our label
        self.label = page.get_label(self.obj)

    def add_nengo_objects(self, page):
        # create a Node and a Connection so the Node will be given the
        # data we want to show while the model is running.
        with page.model:
            self.node = nengo.Node(self.gather_data,
                                   size_in=self.n_lines)
            synapse = self.page.config[self].synapse
            self.conn = nengo.Connection(self.output, self.node,
                                         synapse=synapse)

    def remove_nengo_objects(self, page):
        # undo the changes made by add_nengo_objects
        page.model.connections.remove(self.conn)
        page.model.nodes.remove(self.node)

    def gather_data(self, t, x):
        """This is the Node function for the Node created in add_nengo_objects
        It will be called by the running model, and will store the data
        that should be sent to the client"""
        self.data.append(self.struct.pack(t, *x))

    def update_client(self, client):
        length = len(self.data)
        if length > 0:
            # we do this slicing because self.gather_data is concurrently
            # appending things to self.data.  This means that self.data may
            # increase in length during this call, so we do the slicing
            # and deletion to maintain thread safety
            item = bytes().join(self.data[:length])
            del self.data[:length]
            client.write_binary(item)

    def javascript(self):
        # generate the javascript that will create the client-side object
        info = dict(uid=id(self), label=self.label,
                    n_lines=self.n_lines)

        json = self.javascript_config(info)
        return 'new Nengo.Value(main, sim, %s);' % json

    def code_python_args(self, uids):
        # generate the list of strings for the .cfg file to save this Component
        # (this is the text that would be passed in to the constructor)
        return [uids[self.obj]]

    def message(self, msg):
        if msg.startswith('synapse:'):
            synapse = float(msg[8:])
            self.page.config[self].synapse = synapse
            self.page.modified_config()
            self.page.sim = None

    @staticmethod
    def default_output(obj):
        """Find default output object for the input object if it exists"""
        output = None
        if isinstance(obj, spa.module.Module):
            if 'default' in obj.outputs.keys():
                output = obj.outputs['default'][0]
        elif isinstance(obj, nengo.network.Network):
            if hasattr(obj, 'output'):
                output = obj.output
        return output



# Question Answering

# This model shows a simple form of question answering. The goal of 
# this model is to build a network that will output answers to questions 
# based on supplied statements as described in the book. You will bind 
# two features (color and shape) by circular convolution. Then you will provide
# a cue to the model to determine either one of the features by deconvolution.

# **Note: Simplified method of building the model using the spa 
# (semantic pointer architecture) package in Nengo 2.0 is shown 
# in the spa_question.py file in the same folder. 

# This model has parameters as described in the book, with the ensembles having 
# 300 neurons in 20 dimensions and the bind and unbind networks having 70 
# neurons per dimension. The model relies on SPA (Semantic Pointer Architecture)
# methods for charachterizing representation, transformation and control.

# Depending on your computer, this model may run slowly which can be avoided by 
# several ways as discussed in the book. In order to run the model in direct 
# mode (explained in the book), the parameter 'neuron_type' should be set to 
# nengo.Direct() while creating the ensembles.

# As mentioned in the book, the color input will alternate every 0.5 seconds 
# between RED and BLUE. In the same way the shape input alternates between 
# CIRCLE and SQUARE. Thus, the model will bind alternatingly RED * CIRCLE and 
# BLUE * SQUARE for 0.5 seconds each. In parallel you will ask a question from 
# the model by providing a cue which will be used for deconvolving bound 
# semantic pointers to determine an answer. For example, when the cue is CIRCLE 
# the model will respond with RED. The cue will cycle through CIRCLE, RED, 
# SQUARE, and BLUE within one second.

# Press the play button to run the simulation.
# The graph labelled "output" shows that the output is most similar to the semantic 
# pointer bound to the current cue. For example, when RED and CIRCLE are being 
# bound and the cue is CIRCLE, the output is most similar to RED.  This is much
# easier to see in the spa_question.py version.

# Setup the environment
import numpy as np
import nengo
from nengo.spa import Vocabulary

dim=20         # Number of dimensions 
N_input=300    # Number of neurons in population
N_conv=70      # Number of neurons per dimension in bind/unbind populations

rng = np.random.RandomState(7)
vocab = Vocabulary(dimensions=dim, rng=rng, max_similarity=0.1)
model = nengo.Network(label='Question Answering', seed=7)

with model:
    #Ensembles
    A = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='color')
    B = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='shape')
    C = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='cue')
    D = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='bound')
    E = nengo.Ensemble(n_neurons=N_input, dimensions=dim, label='output') 
    
    #Creating the bind network
    bind = nengo.networks.CircularConvolution(n_neurons=N_conv, dimensions=dim)
    nengo.Connection(A, bind.A)
    nengo.Connection(B, bind.B)
    nengo.Connection(bind.output, D) 
    
    #Creating the Unbind network
    unbind = nengo.networks.CircularConvolution(n_neurons=N_conv, 
                                    dimensions=dim, invert_a=True)
    nengo.Connection(C, unbind.A)
    nengo.Connection(D, unbind.B)
    nengo.Connection(unbind.output, E)

    #Getting semantic pointer values
    CIRCLE=vocab.parse('CIRCLE').v
    BLUE=vocab.parse('BLUE').v
    RED=vocab.parse('RED').v
    SQUARE=vocab.parse('SQUARE').v
    ZERO=[0]*dim
    
    #function for providing color input
    def color_input(t):
        if (t // 0.5) % 2 == 0:
            return RED
        else:
            return BLUE
        
    #function for providing shape input
    def shape_input(t):
        if (t // 0.5) % 2 == 0:
            return CIRCLE
        else:
            return SQUARE

    #function for providing the cue
    def cue_input(t):
        sequence = [ZERO, CIRCLE, RED, ZERO, SQUARE, BLUE]
        idx = int((t // (1. / len(sequence))) % len(sequence))
        return sequence[idx]
      
    #Defining inputs
    inputA = nengo.Node(output=color_input, size_out=dim)
    inputB = nengo.Node(output=shape_input, size_out=dim)
    inputC = nengo.Node(output=cue_input, size_out=dim)
    
    #Connecting input to ensembles
    nengo.Connection(inputA, A)
    nengo.Connection(inputB, B)
    nengo.Connection(inputC, C)



import collections

import nengo.spa as spa
try:
    import nengo_spa
except ImportError:
    nengo_spa = None

from nengo_gui.components.component import Component


class SpaPlot(Component):
    """Parent class for pointer.Pointer and spa_similarity.SpaSimilarity"""

    def __init__(self, obj, **kwargs):
        super(SpaPlot, self).__init__()
        self.obj = obj
        self.data = collections.deque()
        self.target = kwargs.get('args', 'default')
        if self.target.startswith('<'):
            target_obj = getattr(obj, self.target[1:-1])
            self.vocab_out = obj.get_output_vocab(target_obj)
        else:
            self.vocab_out = obj.outputs[self.target][1]

    def attach(self, page, config, uid):
        super(SpaPlot, self).attach(page, config, uid)
        self.label = page.get_label(self.obj)
        self.vocab_out.include_pairs = config.show_pairs

    def update_client(self, client):
        while len(self.data) > 0:
            data = self.data.popleft()
            client.write_text(data)

    def code_python_args(self, uids):
        return [uids[self.obj], 'args=%r' % self.target]

    @staticmethod
    def applicable_targets(obj):
        targets = []
        if (isinstance(obj, spa.module.Module) or
            (nengo_spa is not None and isinstance(obj, nengo_spa.Network))):

                if hasattr(obj, 'outputs'):
                    for target_name, (obj, vocab) in obj.outputs.items():
                        if vocab is not None:
                            targets.append(target_name)
                elif hasattr(obj, 'output'):
                    # TODO: check for other outputs than obj.output
                    try:
                        v = obj.get_output_vocab(obj.output)
                        if v is not None:
                            targets.append('<output>')
                    except KeyError:
                        # Module has no output vocab
                        pass

        return targets



import inspect

import nengo

import nengo_gui.components

def make_param(name, default):
    try:
        # the most recent way of making Parameter objects
        p = nengo.params.Parameter(name=name, default=default)
    except TypeError:
        # this was for older releases of nengo (v2.0.3 and earlier)
        p = nengo.params.Parameter(default=default)
    return p

class Config(nengo.Config):
    def __init__(self):
        super(Config, self).__init__()
        for cls in [nengo.Ensemble, nengo.Node]:
            self.configures(cls)
            self[cls].set_param('pos', make_param(name='pos', default=None))
            self[cls].set_param('size', make_param(name='size', default=None))
        self.configures(nengo.Network)
        self[nengo.Network].set_param('pos',
                                      make_param(name='pos', default=None))
        self[nengo.Network].set_param('size',
                                      make_param(name='size', default=None))
        self[nengo.Network].set_param('expanded',
                                      make_param(name='expanded',
                                                 default=False))
        self[nengo.Network].set_param('has_layout',
                                      make_param(name='has_layout',
                                                 default=False))

        for clsname, cls in inspect.getmembers(nengo_gui.components):
            if inspect.isclass(cls):
                if issubclass(cls, nengo_gui.components.component.Component):
                    if cls != nengo_gui.components.component.Component:
                        self.configures(cls)
                        for k, v in cls.config_defaults.items():
                            p = make_param(name=k, default=v)
                            self[cls].set_param(k, p)

    def dumps(self, uids):
        lines = []
        for obj, uid in sorted(uids.items(), key=lambda x: x[1]):

            if isinstance(obj, (nengo.Ensemble, nengo.Node, nengo.Network)):
                if self[obj].pos is not None:
                    lines.append('_viz_config[%s].pos=%s' % (uid,
                                                             self[obj].pos))
                if self[obj].size is not None:
                    lines.append('_viz_config[%s].size=%s' % (uid,
                                                              self[obj].size))
                if isinstance(obj, nengo.Network):
                    lines.append('_viz_config[%s].expanded=%s'
                                 % (uid, self[obj].expanded))
                    lines.append('_viz_config[%s].has_layout=%s'
                                 % (uid, self[obj].has_layout))

            elif isinstance(obj, nengo_gui.components.editor.Editor):
                pass  # Do not persist editor

            elif isinstance(obj, nengo_gui.components.component.Component):
                lines.append('%s = %s' % (uid, obj.code_python(uids)))
                for k in obj.config_defaults.keys():
                    v = getattr(self[obj], k)
                    val = repr(v)

                    try:
                        recovered_v = eval(val, {})
                    except:
                        raise ValueError("Cannot save %s to config. Only "
                                         "values that can be successfully "
                                         "evaluated are allowed." % (val))

                    if recovered_v != v:
                        raise ValueError("Cannot save %s to config, recovery "
                                         "failed. Only "
                                         "values that can be recovered after "
                                         "being entered into the config file " 
                                         "can be saved." % (val))

                    lines.append('_viz_config[%s].%s = %s' % (uid, k, val))

        return '\n'.join(lines)



# Tutorial 23: Binding Concepts

# We now show that you can combine semantic pointers together to store
# structured information.  One of the standard problems in cognitive science
# is the Binding Problem: how do different concepts get represented and
# associated together.  For example, if I think about a red circle and and
# a blue triangle, how does my mind connect RED to CIRCLE and BLUE to TRIANGLE,
# and how does it keep those separate?  Notice that we can't simply add
# RED + CIRCLE + BLUE + TRIANGLE since that would give the same result as
# RED + TRIANGLE + BLUE + CIRCLE, meaning that we couldn't distinguish
# "red circle and blue triangle" from "red triangle and blue circle".
#
# With the semantic pointer approach, we solve this problem by using the
# mathematical operation of circular convolution.  Like addition, this
# operation combines two vectors into a new vector.  However, unlike addition,
# this new vector is very different from both of the original vectors.  (Note
# that this operation was first explored for this purpose by Tony Plate's in
# his Holographic Reduced Representations: "Tony A. Plate. Holographic Reduced
# Representations. IEEE Transactions on Neural Networks, (6):3:623-641, 1995")
# If we use "*" to indicate circular convolution, then we can solve the above
# problem by storing RED * CIRCLE + BLUE * TRIANGLE
#
# Since circular convolution can be implemented with multiplication and linear
# transformation, neurons can readily implement this operation.  In this
# tutorial, we have two inputs (shape and color) and whatever vector is
# stored in these inputs will be convolved together and fed into a memory.
# This is realised by the spa.Cortical module, which directs the flow of
# actions in the cortex.
#
# Try setting the color to RED and the State to CIRCLE.  The memory should
# gradually build up to contain RED*CIRCLE (or, equivalently, CIRCLE*RED).  Now
# clear the inputs (by doing Set value and leaving them blank).  The memory
# should continue ot hold RED*CIRCLE.  Now set the inputs to BLUE and TRIANGLE.
# The memory should gradually change so that it also contains BLUE*TRIANGLE. If
# you feed in these new inputs for a long time, the old RED*CIRCLE memory will
# eventually disappear.  However, if you clear the value being input when both
# values are in memory, it should maintain both bound concepts in memory at the
# same time.
#
# As with all semantic pointer models, the accuracy and capacity of this memory
# will increase as you increase dimensionality (but the simulation will take
# longer to run).  We have argued that around 800 dimensions are sufficient for
# human working memory capacity.

import nengo
import nengo.spa as spa

D = 32  # the dimensionality of the vectors

model = spa.SPA()
with model:
    model.color = spa.State(D)
    model.shape = spa.State(D)
    model.memory = spa.State(D, feedback=1)

    actions = spa.Actions(
        'memory = color * shape',
        )

    model.cortical = spa.Cortical(actions)



# Learning a communication channel

# This model shows how to include synaptic plasticity in Nengo models using the 
# hPES learning rule. Here we implement error-driven learning to learn to 
# compute a simple communication channel. This is done by using an error signal  
# provided by a neural ensemble to modulate the connection between two other 
# ensembles.

# The model has parameters as described in the book. Note that hPES learning 
# rule is built into Nengo 1.4 as mentioned in the book. The same rule can be 
# implemented in Nengo 2.0 by combining the PES and the BCM rule as shown in the
# code. Also, instead of using the 'gate' and 'switch' as described in the book,
# 'inhibit' population is used which serves the same purpose of turning off the 
# learning by inhibiting the error population. For computing the actual error 
# (which is required only for analysis), direct mode can be used by specifying 
# the neuron_type as 'nengo.Direct()' while creating the 'actual_error' ensemble.

# When you run the model, you will see that the 'post' population gradually 
# learns to compute the communication channel. In the model, you will inhibit 
# the error population after 15 seconds to turn off learning and you will see 
# that the 'post' population will still track the 'pre' population showing that 
# the model has actually learned the input.

# Press the play button to run the simulation.
# From the plots, you will see that the post population initially doesn't track 
# the pre population but eventually it learns the function (which is a 
# communication channel in this case) and starts tracking the post population. 
# After the error is inhibited (i.e., t>15s, shown by value zero in the error 
# plot), the post population continues to track the pre population. The 
# actual_error graph shows that there is significant error between the pre and 
# the post populations at the begining which eventually gets reduced to almost 
# zero as model learns the communication channel.

# The model can also learn other functions by using an appropriate error signal.
# For example to learn a square function, comment out the lines marked 
# 'Communication Channel' and uncomment the lines marked 'Square' in the code. 
# Run the model again and you will see that the model successfully learns the 
# square function.

#Setup the environment
import numpy as np
import nengo
from nengo.processes import WhiteSignal

model = nengo.Network(label='Learning', seed=8)
with model:
    #Ensembles to represent populations
    pre = nengo.Ensemble(50, dimensions=1)
    post = nengo.Ensemble(50, dimensions=1)
    error = nengo.Ensemble(100, dimensions=1)
    actual_error = nengo.Ensemble(100, dimensions=1, neuron_type=nengo.Direct())
    
    #Actual Error = pre - post (direct mode)
    #Square
    #nengo.Connection(pre, actual_error, function=lambda x: x**2, transform=-1)   
    nengo.Connection(pre, actual_error, transform=-1)   #Communication Channel
    nengo.Connection(post, actual_error, transform=1)
     
    #Error = pre - post
    #Square
    #nengo.Connection(pre, error, function=lambda x: x**2, transform=-1)
    #Communication Channel
    nengo.Connection(pre, error, transform=-1, synapse=0.02)
    nengo.Connection(post, error, transform=1, synapse=0.02)
    
    #Connecting pre population to post population (communication channel)
    conn = nengo.Connection(pre, post, function=lambda x: np.random.random(1),
                            solver=nengo.solvers.LstsqL2(weights=True))
    
    #Adding the learning rule to the connection 
    conn.learning_rule_type ={'my_pes': nengo.PES(learning_rate=1e-3), 
                                        'my_bcm': nengo.BCM()}

    #Error connections don't impart current
    error_conn = nengo.Connection(error, conn.learning_rule['my_pes'])
    
    #Providing input to the model
    input = nengo.Node(WhiteSignal(30, high=10))     # RMS = 0.5 by default
    # Connecting input to the pre ensemble
    nengo.Connection(input, pre, synapse=0.02)  
    
    #function to inhibit the error population after 25 seconds
    def inhib(t):
        return 2.0 if t > 15.0 else 0.0
    
    #Connecting inhibit population to error population
    inhibit = nengo.Node(inhib)
    nengo.Connection(inhibit, error.neurons, transform=[[-1]] * error.n_neurons, 
                                                synapse=0.01)
                                                
                                                


"""Nengo GUI backend implementation."""

from __future__ import print_function

import hashlib
import json
import logging
import mimetypes
import os
import os.path
import pkgutil
try:
    from urllib.parse import unquote
except ImportError:  # Python 2.7
    from urllib import unquote
try:
    import ssl
except ImportError:  # for Python without ssl support
    from . import nossl as ssl
import time

import nengo_gui
from nengo_gui.completion import get_completions
import nengo_gui.exec_env
import nengo_gui.page
from nengo_gui import server
from nengo_gui.password import checkpw, gensalt


logger = logging.getLogger(__name__)


class Session(object):
    __slots__ = ['creation_time', 'authenticated', 'peer_name', 'login_host']

    def __init__(self):
        self.creation_time = time.time()
        self.authenticated = False
        self.peer_name = None
        self.login_host = None


class SessionManager(object):
    def __init__(self, time_to_live):
        self.time_to_live = time_to_live
        self._sessions = {}

    def __getitem__(self, session_id):
        session = self._sessions.get(session_id, None)
        if (session is None or
                session.creation_time + self.time_to_live < time.time()):
            del self._sessions[session_id]
            raise KeyError("Session expired")
        return session

    def __len__(self):
        return len(self._sessions)

    def add_session(self, request, session):
        session_id = self._new_session_id(request)
        self._sessions[session_id] = session
        return session_id

    def _new_session_id(self, request):
        return gensalt(16)


class RequireAuthentication(object):
    def __init__(self, login_page):
        self.login_page = login_page

    def get_token(self, inst):
        if 'token' in inst.db:
            return inst.db['token']
        elif inst.headers.get('Authorization', '').lower().startswith(
                'token '):
            return inst.headers.get('Authorization').split(' ')[1]
        return None

    def __call__(self, fn):
        def auth_checked(inst):
            session = inst.get_session()
            if session.authenticated:
                return fn(inst)
            elif inst.server.verify_token(self.get_token(inst)):
                self.authenticate(inst)
                return fn(inst)
            return server.HttpRedirect(self.login_page)
        return auth_checked

    @classmethod
    def authenticate(cls, request_handler):
        session = request_handler.get_session()
        session.authenticated = True
        session.login_host = request_handler.headers.get('host', None)
        try:
            session.peer_name = request_handler.request.getpeername()[0]
        except Exception:
            logger.warning(
                "Cannot get peer name. Session will not be tied to "
                "client.", exc_info=True)
        request_handler.persist_session(session)


class GuiRequestHandler(server.HttpWsRequestHandler):
    http_commands = {
        '/': 'serve_main',
        '/login': 'login_page',
        '/static': 'serve_static',
        '/browse': 'browse',
        '/complete': 'complete',
        '/favicon.ico': 'serve_favicon',
    }

    def get_expected_origins(self):
        login_host = self.get_session().login_host
        return [login_host] if login_host is not None else []

    def login_page(self):
        session = self.get_session()
        content = b''

        if session.authenticated:
            return server.HttpRedirect('/')

        if 'pw' in self.db:
            valid_pw = (
                self.server.settings.password_hash is not None and
                checkpw(self.db['pw'], self.server.settings.password_hash))
            valid_token = self.server.verify_token(self.db['pw'])
            if valid_pw or valid_token:
                RequireAuthentication.authenticate(self)
                return server.HttpRedirect('/')
            else:
                content += b'<p><strong>Invalid password. Try again.'
                content += b'</strong></p>'
        else:
            content += b'<p>Please enter the password or the security token '
            content += b'shown by Nengo when it was started:</p>'

        return server.HtmlResponse(content + b'''
            <form method="POST"><p>
                <label for="pw">Password: </label>
                <input type="password" name="pw" />
                <input type="submit" value="Log in" />
            </p></form>
        ''')

    @RequireAuthentication('/login')
    def serve_static(self):
        """Handles http://host:port/static/* by returning pkg data"""
        assert self.resource[0] == '/'
        static_dir = 'static' + os.sep
        fn = os.path.normpath(self.resource[1:])
        if os.path.commonprefix((static_dir, fn)) != static_dir:
            raise server.Forbidden()
        mimetype, encoding = mimetypes.guess_type(fn)
        data = pkgutil.get_data('nengo_gui', fn)
        return server.HttpResponse(data, mimetype)

    @RequireAuthentication('/login')
    def browse(self):
        r = [b'<ul class="jqueryFileTree" style="display: none;">']
        d = unquote(self.db['dir'])
        ex_tag = '//examples//'
        ex_html = b'<em>built-in examples</em>'
        if d == '.':
            r.append(b'<li class="directory collapsed examples_dir">'
                     b'<a href="#" rel="' + ex_tag.encode('utf-8') + b'">' +
                     ex_html + b'</a></li>')
            path = '.'
        elif d.startswith(ex_tag):
            path = os.path.join(nengo_gui.__path__[0],
                                'examples', d[len(ex_tag):])
        else:
            path = os.path.join('.', d)

        for f in sorted(os.listdir(path)):
            ff = os.path.join(path, f).encode('utf-8')
            if os.path.isdir(os.path.join(path, f)):
                f = f.encode('utf-8')
                r.append(b'<li class="directory collapsed">'
                         b'<a href="#" rel="' + ff + b'/">' + f + b'</a></li>')
            else:
                e = os.path.splitext(f)[1][1:]  # get .ext and remove dot
                if e == 'py':
                    e = e.encode('utf-8')
                    f = f.encode('utf-8')
                    r.append(b'<li class="file ext_' + e + b'">'
                             b'<a href="#" rel="' + ff + b'">' +
                             f + b'</a></li>')
        r.append(b'</ul>')
        return server.HtmlResponse(b''.join(r))

    @RequireAuthentication('/login')
    def complete(self):
        completions = get_completions(
            self.db['code'], int(self.db['row']) + 1, int(self.db['col']),
            self.db['filename'])
        return server.JsonResponse([{
            'name': c.name,
            'value': c.name,
            'score': 1,
            'meta': c.type,
        } for c in completions])

    @RequireAuthentication('/login')
    def serve_main(self):
        if self.resource != '/':
            raise server.InvalidResource(self.resource)

        filename = self.query.get('filename', [None])[0]
        reset_cfg = self.query.get('reset', [False])[0]
        page = self.server.create_page(filename, reset_cfg=reset_cfg)

        # read the template for the main page
        html = pkgutil.get_data('nengo_gui', 'templates/page.html')
        if isinstance(html, bytes):
            html = html.decode("utf-8")

        # fill in the javascript needed and return the complete page
        components = page.create_javascript()
        data = (html % dict(components=components)).encode('utf-8')
        return server.HttpResponse(data)

    def serve_favicon(self):
        self.resource = '/static/favicon.ico'
        return self.serve_static()

    @RequireAuthentication('/login')
    def ws_default(self):
        """Handles ws://host:port/viz_component with a websocket"""
        # figure out what component is being connected to

        gui = self.server
        uid = int(self.query['uid'][0])

        component = gui.component_uids[uid]
        while self.ws.state is server.WebSocket.ST_OPEN:
            try:
                if component.replace_with is not None:
                    component.finish()
                    component = component.replace_with

                # read all data coming from the component
                msg = self.ws.read_frame()
                while msg is not None:
                    if not self._handle_ws_msg(component, msg):
                        return
                    msg = self.ws.read_frame()

                # send data to the component
                component.update_client(self.ws)
                component.page.save_config(lazy=True)
                time.sleep(0.01)
            except server.SocketClosedError:
                # This error means the server has shut down
                component.page.save_config(lazy=False)  # Stop nicely
                break
            except:
                logger.exception("Error during websocket communication.")

        # After hot loop
        component.finish()

    def _handle_ws_msg(self, component, msg):
        """Handle websocket message. Returns True when further messages should
        be handled and false when no further messages should be processed."""
        if msg.data.startswith('config:'):
            return self._handle_config_msg(component, msg)
        elif msg.data.startswith('remove'):
            return self._handle_remove_msg(component, msg)
        else:
            try:
                component.message(msg.data)
                return True
            except:
                logging.exception('Error processing: %s', repr(msg.data))

    def _handle_config_msg(self, component, msg):
        cfg = json.loads(msg.data[7:])
        old_cfg = {}
        for k in component.config_defaults.keys():
            v = getattr(
                component.page.config[component], k)
            old_cfg[k] = v
        if not cfg == old_cfg:
            # Register config change to the undo stack
            component.page.config_change(
                component, cfg, old_cfg)
        for k, v in cfg.items():
            setattr(
                component.page.config[component],
                k, v)
        component.page.modified_config()
        return True

    def _handle_remove_msg(self, component, msg):
        if msg.data != 'remove_undo':
            # Register graph removal to the undo stack
            component.page.remove_graph(component)
        component.page.remove_component(component)
        component.page.modified_config()
        return False

    def get_session(self):
        try:
            peer_name = self.request.getpeername()[0]
        except Exception:
            peer_name = None

        try:
            session_id = self.cookie['_session_id'].value
            session = self.server.sessions[session_id]
            if session.peer_name != peer_name:
                logger.warning(
                    "Session peer name mismatch: %s, %s", peer_name,
                    session.peer_name)
                raise KeyError()
        except KeyError:
            session = Session()
        return session

    def persist_session(self, session):
        session_id = self.server.sessions.add_session(self.request, session)
        self.cookie['_session_id'] = session_id

    def log_message(self, format, *args):
        logger.info(format, *args)


class ModelContext(object):
    """Provides context information to a model. This can include the locals
    dictionary, the filename and whether this model can (or is allowed) to be
    written to disk."""

    __slots__ = ['model', 'filename', 'locals', 'writeable']

    def __init__(self, model=None, locals=None, filename=None, writeable=True):
        self.filename = filename
        if self.filename is not None:
            try:
                self.filename = os.path.relpath(filename)
            except ValueError:
                # happens on Windows if filename is on a different
                # drive than the current directory
                self.filename = filename
            self.writeable = writeable
        else:
            self.writeable = False

        if model is None and locals is not None:
            model = locals.get('model', None)

        if model is None and filename is None:
            raise ValueError("No model.")

        self.model = model
        self.locals = locals


class GuiServerSettings(object):
    __slots__ = [
        'listen_addr',
        'auto_shutdown',
        'password_hash',
        'ssl_cert',
        'ssl_key',
        'session_duration',
    ]

    def __init__(
            self, listen_addr=('localhost', 8080), auto_shutdown=2,
            password_hash=None, ssl_cert=None, ssl_key=None,
            session_duration=60 * 60 * 24 * 30):
        self.listen_addr = listen_addr
        self.auto_shutdown = auto_shutdown
        self.password_hash = password_hash
        self.ssl_cert = ssl_cert
        self.ssl_key = ssl_key
        self.session_duration = session_duration

    @property
    def use_ssl(self):
        if self.ssl_cert is None and self.ssl_key is None:
            return False
        elif self.ssl_cert is not None and self.ssl_key is not None:
            return True
        else:
            raise ValueError("SSL needs certificate file and key file.")


class GuiServer(server.ManagedThreadHttpServer):
    def __init__(
            self, model_context, server_settings=GuiServerSettings(),
            page_settings=nengo_gui.page.PageSettings()):
        if nengo_gui.exec_env.is_executing():
            raise nengo_gui.exec_env.StartedGUIException()
        self.settings = server_settings

        server.ManagedThreadHttpServer.__init__(
            self, self.settings.listen_addr, GuiRequestHandler)
        if self.settings.use_ssl:
            for b in self.bindings:
                b.socket = ssl.wrap_socket(
                    b.socket, certfile=self.settings.ssl_cert,
                    keyfile=self.settings.ssl_key, server_side=True)

        self.auth_token = gensalt(24)
        self._one_time_auth_tokens = set()
        self.sessions = SessionManager(self.settings.session_duration)

        # the list of running Pages
        self.pages = []

        # a mapping from uids to Components for all running Pages.
        # this is used to connect the websockets to the appropriate Component
        self.component_uids = {}

        self.model_context = model_context
        self.page_settings = page_settings

        self._last_access = time.time()

    def create_page(self, filename, reset_cfg=False):
        """Create a new Page with this configuration"""
        page = nengo_gui.page.Page(
            self, filename=filename, settings=self.page_settings,
            reset_cfg=reset_cfg)
        self.pages.append(page)
        return page

    def remove_page(self, page):
        self._last_access = time.time()
        self.pages.remove(page)
        if (not self._shutting_down and self.settings.auto_shutdown > 0 and
                len(self.pages) <= 0):
            time.sleep(self.settings.auto_shutdown)
            earliest_shutdown = self._last_access + self.settings.auto_shutdown
            if earliest_shutdown < time.time() and len(self.pages) <= 0:
                logging.info(
                    "No connections remaining to the nengo_gui server.")
                self.shutdown()

    def verify_token(self, token):
        if token in self._one_time_auth_tokens:
            self._one_time_auth_tokens.remove(token)
            return True
        elif token == self.auth_token:
            return True
        else:
            return False

    def gen_one_time_token(self):
        token = gensalt(24)
        self._one_time_auth_tokens.add(token)
        return token



# Tutorial 12: Differential Equations

# Recurrent connections can be used to implement not just memory, as in the
# previous tutorial, but also any differential equation.

# For example, the differential equation for a low-pass filter (a system where
# the output y is a smoothed version of the input x) is 
#     dy/dt = x/tau - y/tau
# where tau is the time constant for the smoothing (larger means smoother)

# To implement this in nengo, we need an input Connection that computes the
# x/tau part, and a recurrent Connection that computes the -y/tau part.

# Because these functions are implemented by neurons, the time constant of
# the synapse itself turns out to be very important here.  We have to take into
# account what synapse is being used when making this connection.

# While the proof is outside the scope of this tutorial, the resulting rule
# is that both your Connections need to get scaled by the synapse value, and
# your recurrent Connection must also add the stored value back in.  That is,
# in this case, the input part of the equation becomes
#      (x/tau) * synapse
# and the recurrent part becomes
#      (-y/tau) * synapse + y
# If we tell Nengo to implement those two functions, we will get the desired
# differential equation.

# Nengo Tip:  In this particular case, those two functions are both linear
# functions, and so we could implement them much more easily using the
# "transform=" approach (see tutorial 10).  This is left as an exercise to the
# user.

# Try running the model and seeing that y is a slowed-down, smoother version
# of x.  What happens if you change the input up and down quickly?  What
# happens with tau=0.1?  What about tau=0.01?

import nengo

model = nengo.Network()
with model:
    stim_x = nengo.Node(0)
    x = nengo.Ensemble(n_neurons=50, dimensions=1)
    nengo.Connection(stim_x, x)
    
    y = nengo.Ensemble(n_neurons=50, dimensions=1)
    tau = 0.5
    synapse = 0.1
    
    def input_function(x):
        return x / tau * synapse
    def recurrent_function(y):
        return (-y / tau) * synapse + y
    
    
    nengo.Connection(x, y, synapse=synapse, function=input_function)
    nengo.Connection(y, y, synapse=synapse, function=recurrent_function)
    



# Question Answering with Memory using SPA

# Now we will build this model again, using the spa (semantic pointer 
# architecture) package built into Nengo 2.0.

# Press the play button to run the simulation.
# Graphs show the colour, shape and cue inputs. The last graph
# shows that the output is most similar to the semantic pointer which was 
# initially bound to the given cue. For example, when SQUARE is 
# provided as a cue, the output is most similar to BLUE.

import nengo
import nengo.spa as spa
from nengo.spa import Vocabulary
import numpy as np

D = 32  # the dimensionality of the vectors
rng = np.random.RandomState(7)
vocab = Vocabulary(dimensions=D, rng=rng, max_similarity=0.1)

#Adding semantic pointers to the vocabulary
CIRCLE=vocab.parse('CIRCLE')
BLUE=vocab.parse('BLUE')
RED=vocab.parse('RED')
SQUARE=vocab.parse('SQUARE')
ZERO=vocab.add('ZERO', [0]*D)

model = spa.SPA(label="Question Answering with Memory", vocabs=[vocab])
with model:
    
    model.A = spa.State(D, label="color")
    model.B = spa.State(D, label="shape")
    model.C = spa.State(D, label="cue")
    model.D = spa.State(D, label="bound")
    model.E = spa.State(D, label="output")
    model.memory = spa.State(D, feedback=1, label="memory")

    actions = spa.Actions(
        'D = A * B',
        'memory = D',
        'E = memory * ~C'
        )
 
    model.cortical = spa.Cortical(actions)
    
    #function for providing color input
    def color_input(t):
        if t < 0.25:
            return 'RED'
        elif t < 0.5:
            return 'BLUE'
        else:
            return 'ZERO'

    #function for providing shape input
    def shape_input(t):
        if t < 0.25:
            return 'CIRCLE'
        elif t < 0.5:
            return 'SQUARE'
        else:
            return 'ZERO'

    #function for providing the cue
    def cue_input(t):
        if t < 0.5:
            return 'ZERO'
        sequence = ['ZERO', 'CIRCLE', 'RED', 'ZERO', 'SQUARE', 'BLUE']
        idx = int(((t - 0.5) // (1. / len(sequence))) % len(sequence))
        return sequence[idx]

    
    #Inputs
    model.input = spa.Input(A=color_input, B=shape_input, C=cue_input)   


# Nengo Example: Controlled Integrator
#
# A controlled integrator is a circuit that acts on two signals:
#
# 1. Input - the signal being integrated
# 2. Control - the control signal to the integrator
#
# A controlled integrator accumulates input, but its state can be directly
# manipulated by the control signal. 
#
# We can use standard network-creation commands to begin creating our
# controlled integrator. We create a Network, and then we create a population
# of neurons (called an *ensemble*). This population of neurons will
# represent the state of our integrator, and the connections between
# the neurons in the ensemble will define the dynamics of our integrator.

import nengo

model = nengo.Network()
with model:
    # Make a population with 200 LIF neurons representing a 2 dimensional
    # signal, with a larger radius to accommodate large inputs
    a = nengo.Ensemble(n_neurons=200, dimensions=2, radius=1.5)

    # Define an input signal within our model
    stim = nengo.Node(0)

    # Connect the Input signal to ensemble a, affecting only the
    # first dimension. The `transform` argument scales the effective
    # strength of the connection by tau.
    tau = 0.1
    nengo.Connection(stim, a[0], transform=tau, synapse=tau)

    control = nengo.Node(1)

    # Connect the "Control" signal to the second of a's two input channels.
    nengo.Connection(control, a[1], synapse=0.005)

    # Create a recurrent connection that first takes the product
    # of both dimensions in A (i.e., the value times the control)
    # and then adds this back into the first dimension of A using
    # a transform
    nengo.Connection(a, a[0],
                     function=lambda x: x[0] * x[1],
                     synapse=tau)


import nengo
import nengo.utils.numpy as npext
import numpy as np


def define_all_seeds(net, seeds=None):
    if seeds is None:
        seeds = {}

    if net.seed is None:
        if net not in seeds:
            # this only happens at the very top level
            seeds[net] = np.random.randint(npext.maxint)
        rng = np.random.RandomState(seed=seeds[net])
    else:
        rng = np.random.RandomState(seed=net.seed)

    # let's use the same algorithm as the builder, just to be consistent
    sorted_types = sorted(net.objects, key=lambda t: t.__name__)
    for obj_type in sorted_types:
        for obj in net.objects[obj_type]:
            # generate a seed for each item, so that manually setting a seed
            #  for a particular item doesn't change the generated seed for
            #  other items
            generated_seed = rng.randint(npext.maxint)
            if obj.seed is None:
                seeds[obj] = generated_seed

    for subnet in net.networks:
        define_all_seeds(subnet, seeds)

    return seeds



import json
import os

from nengo_gui.components.editor import Editor
import nengo_gui.exec_env


class AceEditor(Editor):

    config_defaults = {}

    def __init__(self):
        super(AceEditor, self).__init__()
        self.pending_messages = []

    def attach(self, page, config, uid):
        super(AceEditor, self).attach(page, config, uid)
        self.current_code = page.code
        self.serve_code = True
        self.last_error = None
        self.last_stdout = None

    def update_code(self, code):
        self.current_code = code
        self.serve_code = True

    def update_client(self, client):
        while self.pending_messages:
            client.write_text(self.pending_messages.pop())
        if self.serve_code:
            i = json.dumps({'code': self.current_code})
            client.write_text(i)
            self.serve_code = False
        if nengo_gui.exec_env.is_executing():
            return
        error = self.page.error
        stdout = self.page.stdout
        if error != self.last_error or stdout != self.last_stdout:
            if error is None:
                short_msg = None
            else:
                if '\n' in error['trace']:
                    short_msg = error['trace'].rsplit('\n', 2)[-2]
                else:
                    short_msg = error['trace']
            client.write_text(json.dumps({'error': error,
                                     'short_msg': short_msg,
                                     'stdout': stdout}))
            self.last_error = error
            self.last_stdout = stdout

    def javascript(self):
        return 'Nengo.ace = new Nengo.Ace("%s", {})' % (id(self),)

    def message(self, msg):
        data = json.loads(msg)
        self.current_code = data['code']

        save_as = data.get('save_as', None)
        if save_as is not None:
            if os.path.exists(save_as):
                msg = ("Could not rename to %s; "
                       "File already exists" % save_as)
                self.pending_messages.append(json.dumps(
                                             {'filename': save_as,
                                              'valid': False,
                                              'error': msg}))
            else:
                try:
                    self.page.filename_cfg = save_as + '.cfg'
                    self.page.save_config(force=True)
                    self.page.filename = save_as
                    with open(self.page.filename, 'w') as f:
                        f.write(self.current_code)
                    self.pending_messages.append(json.dumps(
                                                 {'filename': save_as,
                                                  'valid': True}))
                    self.page.net_graph.update_code(self.current_code)
                except IOError:
                    msg = "Could not save %s; permission denied" % save_as
                    self.pending_messages.append(json.dumps(
                                                 {'filename': save_as,
                                                  'valid': False,
                                                  'error': msg}))
        elif data['save']:
            try:
                with open(self.page.filename, 'w') as f:
                    f.write(self.current_code)
                self.pending_messages.append(json.dumps({
                    'save_success': True}))
            except IOError:
                print("Could not save %s; permission denied" %
                      self.page.filename)
                self.page.net_graph.update_code(self.current_code)
        else:
            self.page.net_graph.update_code(self.current_code)



from nengo_gui.components.component import Component


class Editor(Component):
    config_defaults = {}

    def __init__(self):
        # the IPython integration requires this component to be early
        # in the list
        super(Editor, self).__init__(component_order=-8)

    def update_code(self, msg):
        pass

    def javascript(self):
        return 'Nengo.disable_editor();'


class NoEditor(Editor):
    def __init__(self):
        super(NoEditor, self).__init__()

    def message(self, msg):
        pass



# Tutorial 21: Sequential Semantic Pointer Actions

# In this example, we define a set of actions that follow through a
# repeating sequence (A, B, C, D, E).  This shows that you can define
# actions which affect the performance of later actions.

# In this example we have changed the default value of the optional argument
# feedback_synapse, which is the time constant controlling the exponential
# decay of the postsynaptic potential. Try using longer time constants
# (e.g. 0.1 or 0.5) and observe what changes. How is the stability of
# a memory representation related to the synaptic time constant?

import nengo
import nengo.spa as spa

D = 32  # the dimensionality of the vectors

model = spa.SPA()
with model:
    model.memory = spa.State(D, feedback=1, feedback_synapse=0.01)

    actions = spa.Actions(
        'dot(memory, A) --> memory=B',
        'dot(memory, B) --> memory=C',
        'dot(memory, C) --> memory=D',
        'dot(memory, D) --> memory=E',
        'dot(memory, E) --> memory=A',
        )

    model.bg = spa.BasalGanglia(actions)
    model.thalamus = spa.Thalamus(model.bg)



# Tutorial 22: Controlled Sequence of Semantic Pointer Actions

# Here, we expand on the previous sequence example and define a model
# that instead of starting from A each time, starts from whatever value
# is currently seen.  This shows that you can selectively route information
# from one group of neurons to another, depending on what action is
# selected.
#
# Try setting vision to A (using "Set value" on the vision graph).  The
# memory should go through the sequence (A, B, C, D, E) and repeat.  Now
# set vision to C.  It should do (C, D, E) and repeat.
#
# Note that it takes slightly longer for the model to transfer information
# from vision to memory than it does to just go to the next item in the
# sequence.  This is due to the time needed for the neurons to respond,
# and is consistent with psychological reaction time data.

import nengo
import nengo.spa as spa

D = 32  # the dimensionality of the vectors

model = spa.SPA()
with model:
    model.vision = spa.State(D)
    model.memory = spa.State(D, feedback=1, feedback_synapse=0.01)

    actions = spa.Actions(
        'dot(memory, A) --> memory=B',
        'dot(memory, B) --> memory=C',
        'dot(memory, C) --> memory=D',
        'dot(memory, D) --> memory=E',
        'dot(memory, E) --> memory=vision',
        '0.5 --> memory=vision'
        )

    model.bg = spa.BasalGanglia(actions)
    model.thalamus = spa.Thalamus(model.bg)



# Nengo Example: Inhibitory Gating of Ensembles

# ## Step 1: Create the network
#
# Our model consists of two ensembles (called A and B) that receive inputs from
# a common sine wave signal generator.
#
# Ensemble A is gated using the output of a node, while Ensemble B is gated
# using the output of a third ensemble (C). This is to demonstrate that
# ensembles can be gated using either node outputs, or decoded outputs from
# ensembles.

import nengo

model = nengo.Network()
with model:
    a = nengo.Ensemble(n_neurons=30, dimensions=1)
    b = nengo.Ensemble(n_neurons=30, dimensions=1)
    c = nengo.Ensemble(n_neurons=30, dimensions=1)

    stim = nengo.Node(0)
    inhibition = nengo.Node(0)

    nengo.Connection(stim, a)
    nengo.Connection(stim, b)
    nengo.Connection(inhibition, a.neurons, transform=[[-2.5]] * 30)
    nengo.Connection(inhibition, c)
    nengo.Connection(c, b.neurons, transform=[[-2.5]] * 30)



# Routed Sequencing with Cleanup Memory

# This model extends the routed sequencing model by creating a scalar ensemble 
# 'cleanup' as explained in the book. We project the 'state' on to the 
# 'cleanup' ensemble using a transformation matrix 'pd', specified by the 
# vocabulary vector 'A'. This implies that each component of the state vector 
# (i.e., the cortical element) is multiplied by the corresponding component of 
# the 'A' vector and summed to produce a one dimensional quantity represented by 
# the 'cleanup' ensemble. This operation is similar to the dot product of the 
# state vector and the vocabulary vector 'A'.

# This model has parameters as described in the book, with a one dimensional 
# cleanup ensemble consisting of 100 neurons.

# Press the play button in the visualizer to run the simulation. 
# The graph on the bottom-left shows the semantic pointer representation of the 
# values stored in the state and the response of the cleanup population is shown 
# in the graph on the bottom-right. The graph on the top-right shows the utility 
# (similarity) of the current basal ganglia input (i.e., state) with the 
# possible vocabulary vectors. Since the cleanup operation is similar to a dot 
# product between the state of the cortical element (state) and the defined 
# vocabulary vector 'A', the value of the cleanup population rises only when the
# state (top-right graph) is 'A'.

#Setup the environment
import numpy as np
import nengo
from nengo import spa
from nengo.spa import Vocabulary

#Number of dimensions for the Semantic Pointers
dimensions = 16

#Make a model object with the SPA network
model = spa.SPA(label='Routed_Sequence with cleanupA', seed=12)

with model:
    #Specifying the modules to be used
    model.state = spa.State(dimensions=dimensions, feedback=1, 
                                    feedback_synapse=0.01)
    model.vision = spa.State(dimensions=dimensions) 
    
    # Specify the action mapping
    actions = spa.Actions(
        'dot(vision, START) --> state = vision',
        'dot(state, A) --> state = B',
        'dot(state, B) --> state = C',
        'dot(state, C) --> state = D',
        'dot(state, D) --> state = E',
        'dot(state, E) --> state = A'
    )
    
    #Creating the BG and Thalamus components that confirm to the specified rules
    model.BG = spa.BasalGanglia(actions=actions)
    model.thal = spa.Thalamus(model.BG)
    
    #Change the seed of this RNG to change the vocabulary
    rng = np.random.RandomState(0)
    vocab = Vocabulary(dimensions=dimensions)

    #Create the transformation matrix (pd) and the cleanup ensemble (cleanupA) 
    pd = [vocab['A'].v.tolist()] 
    model.cleanup = spa.State(neurons_per_dimension=100, dimensions=1)
    
    #Function that provides the model with an initial input semantic pointer.
    def start(t):
        if t < 0.4:
            return '0.8*START+D'
        else:
            return '0'

    #Input
    model.input = spa.Input(vision=start)
    
    #Projecting the state of the cortex on to the cleanup ensemble using a 
    #transformation matrix 'pd'.
    nengo.Connection(model.state.output, model.cleanup.input, transform=pd)

